@@ -1,10 +1,10 @@
 /* Copyright 2014. The Regents of the University of California.
- * Copyright 2015-2016. Martin Uecker.
+ * Copyright 2015-2017. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  * 
  * Authors: 
- * 2014-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2014-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014 Jonathan Tamir <jtamir@eecs.berkeley.edu>
  */
 
@@ -20,8 +20,7 @@
 #include "num/init.h"
 #include "num/ops.h"
 
-#include "wavelet2/wavelet.h"
-#include "wavelet3/wavthresh.h"
+#include "wavelet/wavthresh.h"
 
 #include "misc/debug.h"
 #include "misc/misc.h"
@@ -63,19 +62,15 @@ static double bench_generic_matrix_multiply(long dims[DIMS])
 	long dimsX[DIMS];
 	long dimsY[DIMS];
 	long dimsZ[DIMS];
-
+#if 1
 	md_select_dims(DIMS, 2 * 3 + 17, dimsX, dims);	// 1 110 1
 	md_select_dims(DIMS, 2 * 6 + 17, dimsY, dims);	// 1 011 1
 	md_select_dims(DIMS, 2 * 5 + 17, dimsZ, dims);	// 1 101 1
-
-	long strsX[DIMS];
-	long strsY[DIMS];
-	long strsZ[DIMS];
-
-	md_calc_strides(DIMS, strsX, dimsX, CFL_SIZE);
-	md_calc_strides(DIMS, strsY, dimsY, CFL_SIZE);
-	md_calc_strides(DIMS, strsZ, dimsZ, CFL_SIZE);
-
+#else
+	md_select_dims(DIMS, 2 * 5 + 17, dimsZ, dims);	// 1 101 1
+	md_select_dims(DIMS, 2 * 3 + 17, dimsY, dims);	// 1 110 1
+	md_select_dims(DIMS, 2 * 6 + 17, dimsX, dims);	// 1 011 1
+#endif
 	complex float* x = md_alloc(DIMS, dimsX, CFL_SIZE);
 	complex float* y = md_alloc(DIMS, dimsY, CFL_SIZE);
 	complex float* z = md_alloc(DIMS, dimsZ, CFL_SIZE);
@@ -83,12 +78,9 @@ static double bench_generic_matrix_multiply(long dims[DIMS])
 	md_gaussian_rand(DIMS, dimsX, x);
 	md_gaussian_rand(DIMS, dimsY, y);
 
-	md_clear(DIMS, dimsZ, z, CFL_SIZE);
-
-
 	double tic = timestamp();
 
-	md_zfmac2(DIMS, dims, strsZ, z, strsX, x, strsY, y);
+	md_ztenmul(DIMS, dimsZ, z, dimsX, x, dimsY, y);
 
 	double toc = timestamp();
 
@@ -396,7 +388,7 @@ static double bench_zl1norm(long scale)
 }
 
 
-static double bench_wavelet_thresh(int version, long scale)
+static double bench_wavelet(long scale)
 {
 	long dims[DIMS] = { 1, 256 * scale, 256 * scale, 1, 16, 1, 1, 1 };
 	long minsize[DIMS] = { [0 ... DIMS - 1] = 1 };
@@ -404,18 +396,7 @@ static double bench_wavelet_thresh(int version, long scale)
 	minsize[1] = MIN(dims[1], 16);
 	minsize[2] = MIN(dims[2], 16);
 
-	const struct operator_p_s* p;
-
-	switch (version) {
-	case 2:
-		p = prox_wavethresh_create(DIMS, dims, 7, minsize, 1.1, true, false);
-		break;
-	case 3:
-		p = prox_wavelet3_thresh_create(DIMS, dims, 6, minsize, 1.1, true);
-		break;
-	default:
-		assert(0);
-	}
+	const struct operator_p_s* p = prox_wavelet_thresh_create(DIMS, dims, 6, 0u, minsize, 1.1, true);
 
 	complex float* x = md_alloc(DIMS, dims, CFL_SIZE);
 	md_gaussian_rand(DIMS, dims, x);
@@ -432,16 +413,6 @@ static double bench_wavelet_thresh(int version, long scale)
 	return toc - tic;
 }
 
-static double bench_wavelet2(long scale)
-{
-	return bench_wavelet_thresh(2, scale);
-}
-
-static double bench_wavelet3(long scale)
-{
-	return bench_wavelet_thresh(3, scale);
-}
-
 
 enum bench_indices { REPETITION_IND, SCALE_IND, THREADS_IND, TESTS_IND, BENCH_DIMS };
 
@@ -500,8 +471,7 @@ const struct benchmark_s {
 	{ bench_zl1norm,	"l1 norm" },
 	{ bench_copy1,		"copy 1" },
 	{ bench_copy2,		"copy 2" },
-	{ bench_wavelet2,	"wavelet soft thresh" },
-	{ bench_wavelet3,	"wavelet soft thresh" },
+	{ bench_wavelet,	"wavelet soft thresh" },
 };
 
 
