@@ -1,10 +1,10 @@
 /* Copyright 2014. The Regents of the University of California.
- * Copyright 2015. Martin Uecker.
+ * Copyright 2015-2016. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
- * 2014-2015 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2014-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  */
 
 #include <assert.h>
@@ -17,8 +17,8 @@
 #include "misc/opts.h"
 
 
-static const char* usage_str = "-b <bitmask> | <dim1> ... <dimN>";
-static const char* help_str = "Convert between a bitmask and set of dimensions.";
+static const char usage_str[] = "-b <bitmask> | <dim1> ... <dimN>";
+static const char help_str[] = "Convert between a bitmask and set of dimensions.";
 
 
 
@@ -30,7 +30,7 @@ int main_bitmask(int argc, char* argv[])
 
 	const struct opt_s opts[] = {
 
-		{ 'b', false, opt_set, &inverse, "\tdimensions from bitmask" },
+		OPT_SET('b', &inverse, "dimensions from bitmask"),
 	};
 
 	cmdline(&argc, argv, 0, 1000, usage_str, help_str, ARRAY_SIZE(opts), opts);
