@@ -26,7 +26,7 @@ static const char help_str[] = "Absolute value of array (|<input>|).\n";
 
 int main_cabs(int argc, char* argv[])
 {
-	mini_cmdline(argc, argv, 2, usage_str, help_str);
+	mini_cmdline(&argc, argv, 2, usage_str, help_str);
 
 	long dims[DIMS];
 
