@@ -1,29 +1,34 @@
-/* Copyright 2013-2015. The Regents of the University of California.
- * All rights reserved. Use of this source code is governed by 
+/* Copyright 2013-2016. The Regents of the University of California.
+ * Copyright 2016. Martin Uecker.
+ * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2015 Martin Uecker <uecker@eecs.berkeley.edu>
+ * 2012-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2013 Dara Bahri <dbahri123@gmail.com>
- * 2015 Siddharth Iyer <sid8795@gmail.com>
+ * 2015-2016 Siddharth Iyer <sid8795@gmail.com>
  *
  *
  * Uecker M, Lai P, Murphy MJ, Virtue P, Elad M, Pauly JM, Vasanawala SS, Lustig M.
- * ESPIRiT - An Eigenvalue Approach to  Autocalibrating Parallel MRI: Where SENSE 
+ * ESPIRiT - An Eigenvalue Approach to Autocalibrating Parallel MRI: Where SENSE
  * meets GRAPPA. Magn Reson Med, 71:990-1001 (2014)
  *
+ * Iyer S, Ong F, Lustig M.
+ * Towards A Parameter Free ESPIRiT: Soft-Weighting For Robust Coil Sensitivity Estimation.
+ * Presented in the session: "New Frontiers In Image Reconstruction" at ISMRM 2016.
+ * http://www.ismrm.org/16/program_files/O86.htm
+ *
  */
 
 #include <assert.h>
 #include <complex.h>
 #include <math.h>
-#include <stdio.h>
 #include <stdbool.h>
 
 #include "num/multind.h"
 #include "num/fft.h"
 #include "num/flpmath.h"
-#include "num/la.h"
+#include "num/linalg.h"
 #include "num/lapack.h"
 #include "num/casorati.h"
 #include "num/rand.h"
@@ -52,6 +57,10 @@
 #define FLIP
 #endif
 
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+
 
 static void eigen_herm3(int M, int N, float val[M], complex float matrix[N][N]) // ordering might be different to herm2
 {
@@ -69,21 +78,15 @@ static void eigen_herm3(int M, int N, float val[M], complex float matrix[N][N])
 			matrix[i][j] = mout[i][j];
 }
 
-
-
-
-
-
-
 static float scurve(float x)
 {
-        if (x <= -1.)
-                return 0.;
+	if (x <= -1.)
+		return 0.;
 
-        if (x >= 1.)
-                return 1.;
+	if (x >= 1.)
+		return 1.;
 
-        return 0.5 * (1. + 2. * x / (1. + powf(x, 2.)));
+	return 0.5 * (1. + 2. * x / (1. + powf(x, 2.)));
 }
 
 static float crop_weight_function(float crth, float val)
@@ -106,11 +109,9 @@ static void crop_weight(const long dims[DIMS], complex float* ptr, weight_functi
 	long cc = dims[3];
 	long mm = dims[4];
 
-	assert(DIMS >= 5);
+	assert(DIMS > 5);
 	assert(1 == md_calc_size(DIMS - 5, dims + 5));
 
-
-
 	for (long m = 0; m < mm; m++) {
 #pragma omp parallel for
 		for (long k = 0; k < zz; k++) {
@@ -134,7 +135,175 @@ void crop_sens(const long dims[DIMS], complex float* ptr, bool soft, float crth,
 }
 
 
+/**
+ * sure_crop - This determines the crop-threshold to use as described in the talk: "Towards A Parameter
+ *	       Free ESPIRiT: Soft-Weighting For Robust Coil Sensitivity Estimation". This was given at the
+ *	       session: "New Frontiers In Image Reconstruction" at ISMRM 2016.
+ *
+ * Parameters:
+ *	var		- Estimated variance in data.
+ *	evec_dims	- The eigenvector dimensions.
+ *	evec_data	- The eigenvectors.
+ *	eptr		- The eigenvalues.
+ *	calreg_dims     - Dimension of the calibration region.
+ *	calreg	        - Calibration data.
+ */
+static float sure_crop(float var, const long evec_dims[5], complex float* evec_data, complex float* eptr, const long calreg_dims[4], const complex float* calreg)
+{
+	// Must be in ascending order.
+        float start = 0.7;
+        float delta = 0.01;
 
+	long num_cvals = (long) (((1 - delta - start)/delta) + 1);
+	long num_maps = evec_dims[4];
+
+	// Construct low-resolution image
+	long im_dims[5];
+	md_select_dims(5, 15, im_dims, evec_dims);
+	complex float* im = md_alloc(5, im_dims, CFL_SIZE);
+	md_resize_center(5, im_dims, im, calreg_dims, calreg, CFL_SIZE);
+	ifftuc(5, im_dims, FFT_FLAGS, im, im);
+
+	// Temporary vector for crop dimensions
+	long cropdims[5];
+	md_select_dims(5, 15, cropdims, calreg_dims);
+	cropdims[4] = num_maps;
+
+	// Eigenvectors (M)
+	complex float* M = md_alloc(5, evec_dims, CFL_SIZE);
+	md_copy(5, evec_dims, M, evec_data, CFL_SIZE);
+
+	// Temporary eigenvector holder to hold low resolution maps
+	complex float* LM = md_alloc(5, evec_dims, CFL_SIZE);
+	// Temporary holder for projection calreg
+	complex float* TC = md_alloc(5, calreg_dims, CFL_SIZE);
+	// Temporary holder to hold low resolution calib maps
+	complex float* CM = md_alloc(5, cropdims, CFL_SIZE);
+
+	// Eigenvalues (W)
+	long W_dims[5];
+	md_select_dims(5, 23, W_dims, evec_dims);
+	complex float* W = md_alloc(5, W_dims, CFL_SIZE);
+	md_copy(5, W_dims, W, eptr, CFL_SIZE);
+
+	// Place holder for the inner product result
+	complex float* ip = md_alloc(5, W_dims, CFL_SIZE);
+
+	// Place holder for the projection result
+	complex float* proj = md_alloc(5, im_dims, CFL_SIZE);
+
+	// Place holder for divergence term
+	long div_dims[5] = MD_INIT_ARRAY(5, 1);
+	complex float* div = md_alloc(5, div_dims, CFL_SIZE);
+
+	// Calculating strides.
+	long str1_ip[5];
+	long str2_ip[5];
+	long stro_ip[5];
+
+	md_calc_strides(5, str1_ip,   im_dims, CFL_SIZE);
+	md_calc_strides(5, str2_ip, evec_dims, CFL_SIZE);
+	md_calc_strides(5, stro_ip,    W_dims, CFL_SIZE);
+
+	long str1_proj[5];
+	long str2_proj[5];
+	long stro_proj[5];
+
+	md_calc_strides(5, str1_proj,    W_dims, CFL_SIZE);
+	md_calc_strides(5, str2_proj, evec_dims, CFL_SIZE);
+	md_calc_strides(5, stro_proj, im_dims, CFL_SIZE);
+
+	long str1_div[5];
+	long str2_div[5];
+	long stro_div[5];
+
+	md_calc_strides(5, str1_div, evec_dims, CFL_SIZE);
+	md_calc_strides(5, str2_div, evec_dims, CFL_SIZE);
+	md_calc_strides(5, stro_div,  div_dims, CFL_SIZE);
+
+	long tdims_ip[5];
+	long tdims_proj[5];
+
+	for (unsigned int i = 0; i < 5; i++) {
+
+		assert((im_dims[i] == evec_dims[i]) || (1 == im_dims[i]) || (1 == evec_dims[i]));
+		assert(( W_dims[i] == evec_dims[i]) || (1 ==  W_dims[i]) || (1 == evec_dims[i]));
+
+		tdims_ip[i]   = (1 == im_dims[i]) ? evec_dims[i] : im_dims[i];
+		tdims_proj[i] = (1 ==  W_dims[i]) ? evec_dims[i] :  W_dims[i];
+	}
+
+	// Starting parameter sweep with SURE.
+	float minMSE = 0;
+	float estMSE = 0;
+	float optVal = 0;
+	float c	     = 0;
+
+	for (int idx = 0; idx < num_cvals; idx++) {
+
+		estMSE = 0;
+		*div   = 0;
+
+		md_clear(5,      W_dims,   ip, CFL_SIZE);
+		md_clear(5,     im_dims, proj, CFL_SIZE);
+		md_clear(5,   evec_dims,   LM, CFL_SIZE);
+		md_clear(5, calreg_dims,   TC, CFL_SIZE);
+
+		c = start + idx * delta;
+
+		// Cropping
+		crop_weight(evec_dims, M, crop_thresh_function, c, W);
+
+		// Projection (stored in proj)
+		md_zfmacc2(5,   tdims_ip,   stro_ip,   ip,   str1_ip, im,   str2_ip, M);
+		md_zfmac2 (5, tdims_proj, stro_proj, proj, str1_proj, ip, str2_proj, M);
+		
+		// Construct low resolution projection image.
+		fftuc(5, im_dims, FFT_FLAGS, proj, proj);
+		md_resize_center(5, calreg_dims, TC, im_dims, proj, CFL_SIZE);
+		md_resize_center(5, im_dims, proj, calreg_dims, TC, CFL_SIZE);
+		ifftuc(5, im_dims, FFT_FLAGS, proj, proj);
+
+		for (int jdx = 0; jdx < md_calc_size(5, im_dims); jdx++)
+			estMSE += powf(cabsf(im[jdx] - proj[jdx]), 2);
+
+		// Construct low-resolution maps 
+		fftuc(5, evec_dims, FFT_FLAGS, LM, M);
+		md_resize_center(5, cropdims, CM, evec_dims, LM, CFL_SIZE);
+		md_resize_center(5, evec_dims, LM, cropdims, CM, CFL_SIZE);
+		ifftuc(5, evec_dims, FFT_FLAGS, LM, LM);
+
+		// Calculating SURE divergence using low resolution maps
+		md_zfmacc2(5, evec_dims, stro_div, div, str1_div, LM, str2_div, LM);
+
+		estMSE += 2 * var * creal(*div);
+
+		if ((0 == idx) || (estMSE < minMSE)) {
+
+			optVal = c;
+			minMSE = estMSE;
+		}
+	}
+
+	md_free(im);
+	md_free(TC);
+	md_free(CM);
+	md_free(M);
+	md_free(LM);
+	md_free(W);
+	md_free(ip);
+	md_free(proj);
+	md_free(div);
+
+	// Smudge factor is to soften by little bit to improve robustness. This is to account for
+	// the sweeped thresholds possibly having a too large a step size between them and for any 
+	// other inconsistencies.
+	float smudge = 0.99;
+	debug_printf(DP_DEBUG1, "Calculated c: %.3f\n", optVal);
+	debug_printf(DP_DEBUG1, "Smudge: %.3f\n", smudge);
+
+	return smudge * optVal;
+}
 
 
 
@@ -323,7 +492,7 @@ void calone_dims(const struct ecalib_conf* conf, long cov_dims[4], long channels
 
 
 
-const struct ecalib_conf ecalib_defaults = { { 6, 6, 6 }, 0.001, -1, -1., false, false, 0.8, true, false, -1., false, true };
+const struct ecalib_conf ecalib_defaults = { { 6, 6, 6 }, 0.001, -1, -1., false, false, 0.8, true, false, -1., false, true, -1., false};
 
 
 
@@ -386,9 +555,11 @@ void calib2(const struct ecalib_conf* conf, const long out_dims[DIMS], complex f
 		md_zsmul(DIMS, out_dims, out_data, out_data, sqrtf((float)channels));
 	}
 
-	debug_printf(DP_DEBUG1, "Crop maps... (%.2f)\n", conf->crop);
+	float c = (conf->crop > 0) ? conf->crop : sure_crop(conf->var, out_dims, out_data, eptr, calreg_dims, data);
 
-	crop_sens(out_dims, out_data, conf->softcrop, conf->crop, eptr);
+	debug_printf(DP_DEBUG1, "Crop maps... (c = %.2f)\n", c);
+
+	crop_sens(out_dims, out_data, conf->softcrop, c, eptr);
 
 	debug_printf(DP_DEBUG1, "Fix phase...\n");
 
@@ -481,7 +652,7 @@ static int number_of_kernels(const struct ecalib_conf* conf, unsigned int N, con
 }
 
 
-void compute_kernels(const struct ecalib_conf* conf, long nskerns_dims[5], complex float** nskerns_ptr, unsigned int SN, float svals[SN], const long caldims[DIMS], const complex float* caldata)
+void compute_kernels(const struct ecalib_conf* conf, long nskerns_dims[5], complex float** nskerns_ptr, unsigned int SN, float val[SN], const long caldims[DIMS], const complex float* caldata)
 {
 	assert(1 == md_calc_size(DIMS - 5, caldims + 5));
 
@@ -498,49 +669,49 @@ void compute_kernels(const struct ecalib_conf* conf, long nskerns_dims[5], compl
 	complex float* nskerns = md_alloc(5, nskerns_dims, CFL_SIZE);
 	*nskerns_ptr = nskerns;
 
-	complex float (*vec)[N] = xmalloc(N * N * sizeof(complex float));
+	PTR_ALLOC(complex float[N][N], vec);
 
-	assert((NULL == svals) || (SN == N));
-	float* val = (NULL != svals) ? svals : xmalloc(N * FL_SIZE);
+	assert(NULL != val);
+	assert(SN == N);
 
 	debug_printf(DP_DEBUG1, "Build calibration matrix and SVD...\n");
 
 #ifdef CALMAT_SVD
-	calmat_svd(conf->kdims, N, vec, val, caldims, caldata);
+	calmat_svd(conf->kdims, N, *vec, val, caldims, caldata);
 
-        if (conf->weighting)
-		soft_weight_singular_vectors(N, conf->kdims, caldims, val, val);
+	if (conf->weighting)
+		soft_weight_singular_vectors(N, conf->var, conf->kdims, caldims, val, val);
 
 	for (int i = 0; i < N; i++)
 		for (int j = 0; j < N; j++) 
 #ifndef FLIP
-			nskerns[i * N + j] = (vec[j][i]) * (conf->weighting ? val[i] : 1.);
+			nskerns[i * N + j] = ((*vec)[j][i]) * (conf->weighting ? val[i] : 1.);
 #else
-			nskerns[i * N + j] = (vec[j][N - 1 - i]) * (conf->weighting ? val[N - 1 - i] : 1.);
+			nskerns[i * N + j] = ((*vec)[j][N - 1 - i]) * (conf->weighting ? val[N - 1 - i] : 1.);
 #endif
 #else
-	covariance_function(conf->kdims, N, vec, caldims, caldata);
+	covariance_function(conf->kdims, N, *vec, caldims, caldata);
 
 	debug_printf(DP_DEBUG1, "Eigen decomposition... (size: %ld)\n", N);
 
 	// we could apply Nystroem method here to speed it up
 
 	float tmp_val[N];
-	lapack_eig(N, tmp_val, vec);
+	lapack_eig(N, tmp_val, *vec);
 
 	// reverse and square root, test for smaller null to avoid NaNs
 	for (int i = 0; i < N; i++)
 		val[i] = (tmp_val[N - 1 - i] < 0.) ? 0. : sqrtf(tmp_val[N - 1 - i]);
 
-        if (conf->weighting)
-		soft_weight_singular_vectors(N, conf->kdims, caldims, val, val);
+	if (conf->weighting)
+		soft_weight_singular_vectors(N, conf-> var, conf->kdims, caldims, val, val);
 
 	for (int i = 0; i < N; i++)
 		for (int j = 0; j < N; j++) 
 #ifndef FLIP
-			nskerns[i * N + j] = vec[N - 1 - i][j] * (conf->weighting ? val[i] : 1.);	// flip
+			nskerns[i * N + j] = (*vec)[N - 1 - i][j] * (conf->weighting ? val[i] : 1.);	// flip
 #else
-			nskerns[i * N + j] = vec[i][j] * (conf->weighting ? val[N - 1 - i] : 1.);	// flip
+			nskerns[i * N + j] = (*vec)[i][j] * (conf->weighting ? val[N - 1 - i] : 1.);	// flip
 #endif
 #endif
 
@@ -556,10 +727,7 @@ void compute_kernels(const struct ecalib_conf* conf, long nskerns_dims[5], compl
 	nskerns_dims[4] = N - number_of_kernels(conf, N, val);
 #endif
 
-	if (NULL == svals)
-		free(val);
-
-	free(vec);
+	PTR_FREE(vec);
 }
 
 
