@@ -19,6 +19,7 @@
 #include "num/lapack.h"
 
 #include "misc/debug.h"
+#include "misc/misc.h"
 
 #include "calib/calib.h"
 #include "calib/calmat.h"
@@ -91,6 +92,7 @@ static char* file_name(const long kernel_dims[3], const long calreg_dims[4]) {
         CALREG,
         calreg_dims[0], calreg_dims[1], calreg_dims[2], calreg_dims[3],
         DAT);
+    
 
     return name;
 
@@ -111,7 +113,7 @@ static int load_noise_sv(const long kernel_dims[3], const long calreg_dims[4], l
     char* name = file_name(kernel_dims, calreg_dims);
     FILE* fp   = fopen(name, "rb");
 
-    if (NULL == fp) {
+    if (!fp) {
         free(name);
         return 0;
     }
@@ -143,9 +145,8 @@ static void save_noise_sv(const long kernel_dims[3], const long calreg_dims[4],
     FILE* fp   = fopen(name, "wb");
 
     if (!fp) {
-        perror("fopen");
         free(name);
-        exit(1);
+        return;
     }
 
     fwrite(E, sizeof(float), L, fp);
@@ -175,30 +176,37 @@ static void save_noise_sv(const long kernel_dims[3], const long calreg_dims[4],
 static void nsv(const long kernel_dims[3], const long calreg_dims[4], long L, float* E, long num_iters)
 {
     
-    if (1 == load_noise_sv(kernel_dims, calreg_dims, L, E)) {
+
+    if (NULL != getenv("TOOLBOX_PATH") && 1 == load_noise_sv(kernel_dims, calreg_dims, L, E)) {
         return;
     }
 
     debug_printf(DP_DEBUG1, "NOTE: Running simulations to figure out noise singular values.\n");
-    debug_printf(DP_DEBUG1, "      This is done once given a particular kernel size, calibration size and number of coils.\n");
+    debug_printf(DP_DEBUG1, "      The simulation results are saved if TOOLBOX_PATH is set.\n");
 
-    float tmpE[L];
+    long N = kernel_dims[0] * kernel_dims[1] * kernel_dims[2] * calreg_dims[3]; 
+
+    float tmpE[N];
     long T = md_calc_size(4, calreg_dims) * sizeof(complex float);
 
     complex float ncalreg[T];
     noise_calreg(T, ncalreg);
 
-    long N = kernel_dims[0] * kernel_dims[1] * kernel_dims[2] * calreg_dims[3]; 
-    complex float (*cov)[N] = calloc(N * N, sizeof(complex float)); // Place holder. Not used for anything.
-    calmat_svd(kernel_dims, N, cov, E, calreg_dims, ncalreg);
+    PTR_ALLOC(complex float[N][N], vec);
+    covariance_function(kernel_dims, N, *vec, calreg_dims, ncalreg);
+    lapack_eig(N, tmpE, *vec);
 
+    for (int idx = 0; idx < L; idx ++)
+        E[idx] = sqrtf(tmpE[N-idx-1]);
+    
     for (long idx = 0; idx < num_iters - 1; idx ++) {
 
         noise_calreg(T, ncalreg);
-        calmat_svd(kernel_dims, N, cov, tmpE, calreg_dims, ncalreg);
+        covariance_function(kernel_dims, N, *vec, calreg_dims, ncalreg);
+        lapack_eig(N, tmpE, *vec);
 
         for (long jdx = 0; jdx < L; jdx ++) {
-            E[jdx] += tmpE[jdx];
+            E[jdx] += sqrtf(tmpE[N-jdx-1]);
         }
 
     }
@@ -207,8 +215,10 @@ static void nsv(const long kernel_dims[3], const long calreg_dims[4], long L, fl
         E[idx] /= num_iters;
     }
 
-    save_noise_sv(kernel_dims, calreg_dims, L, E);
-    free(cov);
+    if (NULL != getenv("TOOLBOX_PATH"))
+        save_noise_sv(kernel_dims, calreg_dims, L, E);
+
+    PTR_FREE(vec);
 
 }
 
@@ -232,13 +242,24 @@ static float estimate_noise_variance(long L, const float* S, const float* E)
 {
 
     float t = 0.f;
-    long  s = 8; // We fit the last s^th singular values.
+    float c = 0.f; // Counter to avoid zero singular values.
+    long  s = 4;   // We fit the last one s^th singular values.
+
+    int num   = L/s;
+    int start = L - num;
+
+    for (long idx = 0; idx < num; idx ++) {
+
+        if (isnan(S[start + idx]) || S[start + idx] <= 0 || isnan(E[start + idx]) || E[start + idx] <= 0) {
+            break;
+        }
+
+        t += ((float)S[start + idx])/((float)E[start + idx]);
+        c += 1.f;
 
-    for (long idx = L - L/s; idx < L; idx ++) {
-        t += ((float)S[idx])/((float)E[idx]);
     }
 
-    return ((float)(s * s) * (t * t)/(L * L))/1.21; //Scaling down since it works well in practice.
+    return ((t * t)/(c * c))/1.21; //Scaling down since it works well in practice.
 
 }
 
@@ -258,17 +279,19 @@ extern float estvar_calreg(const long kernel_dims[3], const long calreg_dims[4],
             (calreg_dims[2] - kernel_dims[2] + 1),
         calreg_dims[3] * kernel_dims[0] * kernel_dims[1] * kernel_dims[2]};
 
-    // Dimension (or rather, just length) of the vector of singular values.
     long L = calmat_dims[0] > calmat_dims[1] ? calmat_dims[1] : calmat_dims[0];
     long N = calmat_dims[1]; //Number of columns.
 
-
-    // Taking the SVD of the calibration region.
+    float tmpE[N];
     float S[L];
-    complex float (*cov)[N] = calloc(N * N, sizeof(complex float)); // Place holder. Not used for anything.
-    calmat_svd(kernel_dims, N, cov, S, calreg_dims, calreg);
-    free(cov);
 
+    PTR_ALLOC(complex float[N][N], vec);
+    covariance_function(kernel_dims, N, *vec, calreg_dims, calreg);
+    lapack_eig(N, tmpE, *vec);
+
+    for (int idx = 0; idx < L; idx ++)
+        S[idx] = sqrtf(tmpE[N-idx-1]);
+    
     return estvar_sv(L, S, kernel_dims, calreg_dims);
 
 }
