@@ -1,10 +1,10 @@
 /* Copyright 2013-2014. The Regents of the University of California.
- * Copyright 2015. Martin Uecker.
+ * Copyright 2015,2017 Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2015	Martin Uecker <martin.uecker@med.uni-goettingen.de.
+ * 2012-2017	Martin Uecker <martin.uecker@med.uni-goettingen.de.
  * 2013		Jonathan Tamir <jtamir@eecs.berkeley.edu>
  */
 
@@ -39,7 +39,7 @@ static const char help_str[] = "Performs coil compression.";
 int main_cc(int argc, char* argv[])
 {
 	long calsize[3] = { 24, 24, 24 };
-	bool proj = false;
+	bool proj = true;
 	long P = -1;
 	bool all = false;
 	enum cc_type { SCC, GCC, ECC } cc_type = SCC;
@@ -47,6 +47,7 @@ int main_cc(int argc, char* argv[])
 	const struct opt_s opts[] = {
 
 		OPT_LONG('p', &P, "N", "perform compression to N virtual channels"),
+		OPT_CLEAR('M', &proj, "output compression matrix"),
 		OPT_VEC3('r', &calsize, "S", "size of calibration region"),
 		OPT_VEC3('R', &calsize, "", "(size of calibration region)"),
 		OPT_SET('A', &all, "use all data to compute coefficients"),
@@ -59,9 +60,11 @@ int main_cc(int argc, char* argv[])
 
 	num_init();
 
-	if (-1 != P)
-		proj = true;
+	if (-1 == P) {
 
+		debug_printf(DP_WARN, "Use -M to output compression matrix.\n");
+		proj = false;
+	}
 
 	long in_dims[DIMS];
 
