@@ -27,7 +27,7 @@ static const char help_str[] = "Compute complex conjugate.\n";
 
 int main_conj(int argc, char* argv[])
 {
-	mini_cmdline(argc, argv, 2, usage_str, help_str);
+	mini_cmdline(&argc, argv, 2, usage_str, help_str);
 
 	num_init();
 
