@@ -31,7 +31,7 @@ static const char help_str[] = "Copy phase from <input> to <output>.\n";
 
 int main_cpyphs(int argc, char* argv[argc])
 {
-	mini_cmdline(argc, argv, 2, usage_str, help_str);
+	mini_cmdline(&argc, argv, 2, usage_str, help_str);
 
 	num_init();
 
