@@ -12,6 +12,7 @@
 #include "num/multind.h"
 #include "num/casorati.h"
 #include "num/filter.h"
+#include "num/init.h"
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -28,8 +29,8 @@
 #endif
 
 
-static const char* usage_str = "<input> <output>";
-static const char* help_str = "Apply filter.\n";
+static const char usage_str[] = "<input> <output>";
+static const char help_str[] = "Apply filter.\n";
 
 
 int main_filter(int argc, char* argv[])
@@ -45,6 +46,8 @@ int main_filter(int argc, char* argv[])
 
 	cmdline(&argc, argv, 2, 2, usage_str, help_str, ARRAY_SIZE(opts), opts);
 
+	num_init();
+
 
 	long in_dims[DIMS];
 	
