@@ -27,7 +27,7 @@ static const char help_str[] = "Invert array (1 / <input>). The output is set to
 
 int main_invert(int argc, char* argv[])
 {
-	mini_cmdline(argc, argv, 2, usage_str, help_str);
+	mini_cmdline(&argc, argv, 2, usage_str, help_str);
 
 	num_init();
 
