@@ -1,4 +1,4 @@
-/* Copyright 2013-2014. The Regents of the University of California.
+/* Copyright 2013-2017. The Regents of the University of California.
  * Copyright 2016. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
@@ -79,6 +79,20 @@ struct iter_fista_conf {
 extern DEF_TYPEID(iter_fista_conf);
 
 
+struct iter_chambolle_pock_conf {
+
+	INTERFACE(iter_conf);
+
+	unsigned int maxiter;
+	float tau;
+	float sigma;
+	float theta;
+	float decay;
+	float tol;
+	_Bool fast;
+};
+
+extern DEF_TYPEID(iter_chambolle_pock_conf);
 
 struct iter_admm_conf {
 
@@ -117,12 +131,25 @@ struct iter_pocs_conf {
 
 extern DEF_TYPEID(iter_pocs_conf);
 
+struct iter_niht_conf {
+
+	INTERFACE(iter_conf);
+
+	unsigned int maxiter;
+	float tol;
+	_Bool do_warmstart;
+};
+
+extern DEF_TYPEID(iter_niht_conf);
+
 extern const struct iter_conjgrad_conf iter_conjgrad_defaults;
 extern const struct iter_landweber_conf iter_landweber_defaults;
 extern const struct iter_ist_conf iter_ist_defaults;
 extern const struct iter_fista_conf iter_fista_defaults;
 extern const struct iter_admm_conf iter_admm_defaults;
 extern const struct iter_pocs_conf iter_pocs_defaults;
+extern const struct iter_niht_conf iter_niht_defaults;
+extern const struct iter_chambolle_pock_conf iter_chambolle_pock_defaults;
 
 
 italgo_fun_f iter_conjgrad;
