@@ -1,10 +1,10 @@
-/* Copyright 2014-2016. The Regents of the University of California.
+/* Copyright 2014-2017. The Regents of the University of California.
  * Copyright 2016. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
- * 2014-2016	Jonathan Tamir <jtamir@eecs.berkeley.edu>
+ * 2014-2017	Jon Tamir <jtamir@eecs.berkeley.edu>
  * 2016		Martin Uecker <martin.uecker@med.uni-goettingen.de>
  */
 
@@ -17,6 +17,9 @@
 #include "num/flpmath.h"
 #include "num/ops.h"
 #include "num/iovec.h"
+#ifdef USE_CUDA
+#include "num/gpuops.h"
+#endif
 
 #include "linops/linop.h"
 
@@ -55,7 +58,7 @@ struct prox_normaleq_data {
 	long size;
 };
 
-DEF_TYPEID(prox_normaleq_data);
+static DEF_TYPEID(prox_normaleq_data);
 
 /**
  * Proximal function for f(z) = 0.5 || y - A z ||_2^2.
@@ -90,7 +93,7 @@ static void prox_normaleq_fun(const operator_data_t* prox_data, float mu, float*
 
 		} else {
 
-			linop_norm_inv_iter((struct linop_s*)pdata->op, rho, z, b);
+			linop_norm_inv_unchecked(pdata->op, rho, (complex float*)z, (const complex float*)b);
 		}
 
 		md_free(b);
@@ -126,7 +129,7 @@ const struct operator_p_s* prox_normaleq_create(const struct linop_s* op, const
 
 	pdata->size = 2 * md_calc_size(linop_domain(op)->N, linop_domain(op)->dims);
 	pdata->adj = md_alloc_sameplace(1, &(pdata->size), FL_SIZE, y);
-	linop_adjoint_iter((struct linop_s*)op, pdata->adj, (const float*)y);
+	linop_adjoint_unchecked(op, (complex float*)pdata->adj, y);
 
 	return operator_p_create(linop_domain(op)->N, linop_domain(op)->dims, 
 			linop_domain(op)->N, linop_domain(op)->dims, 
@@ -152,7 +155,7 @@ struct prox_leastsquares_data {
 	long size;
 };
 
-DEF_TYPEID(prox_leastsquares_data);
+static DEF_TYPEID(prox_leastsquares_data);
 
 
 /**
@@ -217,7 +220,7 @@ struct prox_l2norm_data {
 	long size;
 };
 
-DEF_TYPEID(prox_l2norm_data);
+static DEF_TYPEID(prox_l2norm_data);
 
 
 /**
@@ -272,7 +275,7 @@ const struct operator_p_s* prox_l2norm_create(unsigned int N, const long dims[N]
  * Data for computing prox_l2ball_fun: 
  * Proximal function for f(z) = Ind{ || y - z ||_2 < eps }
  *
- * @param center y
+ * @param y y
  * @param eps
  * @param size size of z
  */
@@ -280,16 +283,38 @@ struct prox_l2ball_data {
 
 	INTERFACE(operator_data_t);
 
-	const float* center;
+	float* y;
 	float eps;
 
 	long size;
+#ifdef USE_CUDA
+	const float* gpu_y;
+#endif
 };
 
-DEF_TYPEID(prox_l2ball_data);
+static DEF_TYPEID(prox_l2ball_data);
+
+
+#ifdef USE_CUDA
+static const float* get_y(const struct prox_l2ball_data* data, bool gpu)
+{
+	const float* y = data->y;
+
+	if (gpu) {
+
+		if (NULL == data->gpu_y)
+			((struct prox_l2ball_data*)data)->gpu_y = md_gpu_move(1, MD_DIMS(data->size), data->y, FL_SIZE);
+
+		y = data->gpu_y;
+	}
+
+	return y;
+}
+#endif
 
 /**
  * Proximal function for f(z) = Ind{ || y - z ||_2 < eps }
+ * Solution is y + (x - y) * q, where q = eps / norm(x - y) if norm(x - y) > eps, 1 o.w.
  * 
  * @param prox_data should be of type prox_l2ball_data
  * @param mu proximal penalty
@@ -301,8 +326,14 @@ static void prox_l2ball_fun(const operator_data_t* prox_data, float mu, float* z
 	UNUSED(mu);
 	struct prox_l2ball_data* pdata = CAST_DOWN(prox_l2ball_data, prox_data);
 
-	if (NULL != pdata->center)
-		md_sub(1, MD_DIMS(pdata->size), z, x_plus_u, pdata->center);
+#ifdef USE_CUDA
+	const float* y = get_y(pdata, cuda_ondevice(x_plus_u));
+#else
+	const float* y = pdata->y;
+#endif
+
+	if (NULL != y)
+		md_sub(1, MD_DIMS(pdata->size), z, x_plus_u, y);
 	else
 		md_copy(1, MD_DIMS(pdata->size), z, x_plus_u, FL_SIZE);
 
@@ -311,8 +342,8 @@ static void prox_l2ball_fun(const operator_data_t* prox_data, float mu, float* z
 	if (q1 > pdata->eps)
 		md_smul(1, MD_DIMS(pdata->size), z, z, pdata->eps / q1);
 
-	if (NULL != pdata->center)
-		md_add(1, MD_DIMS(pdata->size), z, z, pdata->center);
+	if (NULL != y)
+		md_add(1, MD_DIMS(pdata->size), z, z, y);
 }
 
 static void prox_l2ball_apply(const operator_data_t* _data, float mu, complex float* dst, const complex float* src)
@@ -322,18 +353,28 @@ static void prox_l2ball_apply(const operator_data_t* _data, float mu, complex fl
 
 static void prox_l2ball_del(const operator_data_t* _data)
 {
-	xfree(CAST_DOWN(prox_l2ball_data, _data));
+	struct prox_l2ball_data* data = CAST_DOWN(prox_l2ball_data, _data);
+#ifdef USE_CUDA
+	if (NULL != data->gpu_y) {
+		md_free((void*)data->gpu_y);
+	}
+#endif
+	xfree(data);
 }
 
-const struct operator_p_s* prox_l2ball_create(unsigned int N, const long dims[N], float eps, const complex float* center)
+const struct operator_p_s* prox_l2ball_create(unsigned int N, const long dims[N], float eps, const complex float* y)
 {
 	PTR_ALLOC(struct prox_l2ball_data, pdata);
 	SET_TYPEID(prox_l2ball_data, pdata);
 
-	pdata->center = (const float*)center;
+	pdata->y = (float*)y;
 	pdata->eps = eps;
 	pdata->size = md_calc_size(N, dims) * 2;
 
+#ifdef USE_CUDA
+	pdata->gpu_y = NULL;
+#endif
+
 	return operator_p_create(N, dims, N, dims, CAST_UP(PTR_PASS(pdata)), prox_l2ball_apply, prox_l2ball_del);
 }
 
@@ -406,7 +447,7 @@ struct prox_zero_data {
 	long size;
 };
 
-DEF_TYPEID(prox_zero_data);
+static DEF_TYPEID(prox_zero_data);
 
 /**
  * Proximal function for f(z) = 0
@@ -467,7 +508,7 @@ struct prox_lineq_data {
 	complex float* tmp;
 };
 
-DEF_TYPEID(prox_lineq_data);
+static DEF_TYPEID(prox_lineq_data);
 
 static void prox_lineq_apply(const operator_data_t* _data, float mu, complex float* dst, const complex float* src)
 {
@@ -525,7 +566,7 @@ struct prox_ineq_data {
 	bool positive;
 };
 
-DEF_TYPEID(prox_ineq_data);
+static DEF_TYPEID(prox_ineq_data);
 
 static void prox_ineq_fun(const operator_data_t* _data, float mu, float* dst, const float* src)
 {
@@ -579,6 +620,16 @@ const struct operator_p_s* prox_greq_create(unsigned int N, const long dims[N],
 	return prox_ineq_create(N, dims, b, true);
 }
 
+/*
+ * Proximal function for nonnegative orthant
+ * f(z) = 1{z >= 0}
+ */
+const struct operator_p_s* prox_nonneg_create(unsigned int N, const long dims[N])
+{
+	return prox_ineq_create(N, dims, NULL, true);
+}
+
+
 struct prox_rvc_data {
 
 	INTERFACE(operator_data_t);
@@ -586,7 +637,7 @@ struct prox_rvc_data {
 	long size;
 };
 
-DEF_TYPEID(prox_rvc_data);
+static DEF_TYPEID(prox_rvc_data);
 
 static void prox_rvc_apply(const operator_data_t* _data, float mu, complex float* dst, const complex float* src)
 {
