@@ -9,7 +9,7 @@ extern void grad(unsigned int D, const long dims[__VLA(D)], unsigned int flags,
 extern void grad_op(unsigned int D, const long dims[__VLA(D)], unsigned int flags, complex float* out, const complex float* in);
 extern void grad_adjoint(unsigned int D, const long dims[__VLA(D)], unsigned int flags, complex float* out, const complex float* in);
 
-extern struct linop_s* grad_init(long N, const long dims[__VLA(N)], unsigned int flags);
+extern struct linop_s* linop_grad_create(long N, const long dims[__VLA(N)], unsigned int flags);
 
 #include "misc/cppwrap.h"
 
