@@ -12,7 +12,7 @@
 #include "misc/cppwrap.h"
 #include "misc/types.h"
 
-TYPEID linop_data_s;
+extern TYPEID linop_data_s;
 typedef struct linop_data_s { TYPEID* TYPEID; } linop_data_t;
 
 
@@ -40,7 +40,7 @@ extern struct linop_s* linop_create2(unsigned int ON, const long odims[__VLA(ON)
 				unsigned int IN, const long idims[__VLA(IN)], const long istrs[__VLA(IN)], linop_data_t* data,
 				lop_fun_t forward, lop_fun_t adjoint, lop_fun_t normal, lop_p_fun_t norm_inv, del_fun_t);
 
-extern const void* linop_get_data(const struct linop_s* ptr);
+extern const linop_data_t* linop_get_data(const struct linop_s* ptr);
 
 
 
@@ -59,10 +59,6 @@ extern void linop_pseudo_inv(const struct linop_s* op, float lambda, unsigned in
 			unsigned int SN, const long sdims[__VLA(SN)], const complex float* src);
 
 
-#ifdef BERKELEY_SVN
-// FIXME: will go away, use this to
-extern struct operator_s* linop2operator_compat(struct linop_s* x);
-#endif
 
 extern void linop_forward_unchecked(const struct linop_s* op, complex float* dst, const complex float* src);
 extern void linop_adjoint_unchecked(const struct linop_s* op, complex float* dst, const complex float* src);
@@ -87,10 +83,6 @@ extern struct linop_s* linop_loop(unsigned int D, const long dims[D], struct lin
 extern const struct linop_s* linop_join(unsigned int D, const struct linop_s* a, const struct linop_s* b);
 
 
-extern void linop_forward_iter(void* _o, float* _dst, const float* _src);
-extern void linop_adjoint_iter(void* _o, float* _dst, const float* _src);
-extern void linop_normal_iter(void* _o, float* _dst, const float* _src);
-extern void linop_norm_inv_iter(void* _o, float lambda, float* _dst, const float* _src);
 
 #include "misc/cppwrap.h"
 
