@@ -1,9 +1,9 @@
-/* Copyright 2015-2016. Martin Uecker.
+/* Copyright 2015-2017. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2015-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2015-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  */
 
 #define _GNU_SOURCE
@@ -60,7 +60,9 @@ static const char* trim_space(const char* str)
 
 static bool show_option_p(const struct opt_s opt)
 {
-	return (NULL != opt.descr) && (')' != opt.descr[strlen(opt.descr) - 1]);
+	return     (NULL != opt.descr)
+		&& !(   ('(' == trim_space(opt.descr)[0])
+		     && (')' == opt.descr[strlen(opt.descr) - 1]));
 }
 
 static void print_usage(FILE* fp, const char* name, const char* usage_str, int n, const struct opt_s opts[static n ?: 1])
@@ -103,6 +105,22 @@ static void print_help(const char* help_str, int n, const struct opt_s opts[n ?:
 }
 
 
+static void check_options(int n, const struct opt_s opts[n ?: 1])
+{
+	bool f[256] = { false };
+
+	for (int i = 0; i < n; i++) {
+
+		assert(256 > (unsigned int)opts[i].c);
+
+		if (f[(unsigned int)opts[i].c])
+			error("duplicate option: %c\n", opts[i].c);
+
+		f[(unsigned int)opts[i].c] = true;
+	}
+}
+
+
 static void process_option(char c, const char* optarg, const char* name, const char* usage_str, const char* help_str, int n, const struct opt_s opts[n ?: 1])
 {
 	if ('h' == c) {
@@ -135,6 +153,8 @@ void cmdline(int* argcp, char* argv[], int min_args, int max_args, const char* u
 	int argc = *argcp;
 	char optstr[2 * n + 2];
 
+	check_options(n, opts);
+
 	save_command_line(argc, argv);
 
 	int l = 0;
@@ -194,7 +214,6 @@ void cmdline(int* argcp, char* argv[], int min_args, int max_args, const char* u
 	for (i = optind; i < argc; i++)
 		argv[i - optind + 1] = argv[i];
 
-	argv[i] = NULL;
 	*argcp = argc - optind + 1;
 }
 
@@ -296,9 +315,10 @@ bool opt_select(void* ptr, char c, const char* optarg)
 
 bool opt_subopt(void* _ptr, char c, const char* optarg)
 {
+	UNUSED(c);
 	struct opt_subopt_s* ptr = _ptr;
 
-	process_option(c, optarg, "foo", "usage", "help", ptr->n, ptr->opts);
+	process_option(optarg[0], optarg + 1, "", "", "", ptr->n, ptr->opts);
 	return false;
 }
 
