@@ -9,6 +9,7 @@
 
 #include <stdbool.h>
 #include <complex.h>
+#include <math.h>
 
 #include "num/multind.h"
 #include "num/flpmath.h"
@@ -38,12 +39,12 @@ static const char help_str[] =
 
 int main_nlinv(int argc, char* argv[])
 {
-	bool waterfat = false;
 	bool normalize = true;
 	float restrict_fov = -1.;
-	float csh[3] = { 0., 0., 0. };
 	const char* psf = NULL;
 	struct noir_conf_s conf = noir_defaults;
+	bool out_sens = false;
+	bool scale_im = false;
 
 	const struct opt_s opts[] = {
 
@@ -53,23 +54,37 @@ int main_nlinv(int argc, char* argv[])
 		OPT_FLOAT('f', &restrict_fov, "FOV", ""),
 		OPT_STRING('p', &psf, "PSF", ""),
 		OPT_SET('g', &conf.usegpu, "use gpu"),
+		OPT_SET('S', &scale_im, "Re-scale image after reconstruction"),
 	};
 
 	cmdline(&argc, argv, 2, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
 
+	if (4 == argc)
+		out_sens = true;
+
+
+
 	num_init();
 
 	long ksp_dims[DIMS];
 	complex float* kspace_data = load_cfl(argv[1], DIMS, ksp_dims);
 
+	// SMS
+	if (1 != ksp_dims[SLICE_DIM]) {
+
+		debug_printf(DP_INFO, "SMS-NLINV reconstruction. Multiband factor: %d\n", ksp_dims[SLICE_DIM]);
+		fftmod(DIMS, ksp_dims, SLICE_FLAG, kspace_data, kspace_data); // fftmod to get correct slice order in output
+	}
+
+
+	assert(1 == ksp_dims[MAPS_DIM]);
+
 	long dims[DIMS];
 	md_copy_dims(DIMS, dims, ksp_dims);
 
-	if (waterfat)
-		dims[CSHIFT_DIM] = 2;
 
 	long img_dims[DIMS];
-	md_select_dims(DIMS, FFT_FLAGS|CSHIFT_FLAG, img_dims, dims);
+	md_select_dims(DIMS, FFT_FLAGS|CSHIFT_FLAG|SLICE_FLAG, img_dims, dims);
 
 	long img_strs[DIMS];
 	md_calc_strides(DIMS, img_strs, img_dims, CFL_SIZE);
@@ -83,9 +98,9 @@ int main_nlinv(int argc, char* argv[])
 	long msk_strs[DIMS];
 	md_calc_strides(DIMS, msk_strs, msk_dims, CFL_SIZE);
 
-	complex float* mask; 
-	complex float* norm = md_alloc(DIMS, msk_dims, CFL_SIZE);
-	complex float* sens = ((4 == argc) ? create_cfl : anon_cfl)((4 == argc) ? argv[3] : "", DIMS, ksp_dims);
+	complex float* mask;
+	complex float* norm = md_alloc(DIMS, img_dims, CFL_SIZE);
+	complex float* sens = (out_sens ? create_cfl : anon_cfl)(out_sens ? argv[3] : "", DIMS, ksp_dims);
 
 
 	complex float* pattern = NULL;
@@ -94,8 +109,13 @@ int main_nlinv(int argc, char* argv[])
 	if (NULL != psf) {
 
 		pattern = load_cfl(psf, DIMS, pat_dims);
-
 		// FIXME: check compatibility
+
+		if (-1 == restrict_fov)
+			restrict_fov = 0.5;
+
+		conf.noncart = true;
+
 	} else {
 
 		md_copy_dims(DIMS, pat_dims, img_dims);
@@ -103,37 +123,14 @@ int main_nlinv(int argc, char* argv[])
 		estimate_pattern(DIMS, ksp_dims, COIL_FLAG, pattern, kspace_data);
 	}
 
-
-	if (waterfat) {
-
-		size_t size = md_calc_size(DIMS, msk_dims);
-		md_copy(DIMS, msk_dims, pattern + size, pattern, CFL_SIZE);
-
-		long shift_dims[DIMS];
-		md_select_dims(DIMS, FFT_FLAGS, shift_dims, msk_dims);
-
-		long shift_strs[DIMS];
-		md_calc_strides(DIMS, shift_strs, shift_dims, CFL_SIZE);
-
-		complex float* shift = md_alloc(DIMS, shift_dims, CFL_SIZE);
-
-		unsigned int X = shift_dims[READ_DIM];
-		unsigned int Y = shift_dims[PHS1_DIM];
-		unsigned int Z = shift_dims[PHS2_DIM];
-		
-		for (unsigned int x = 0; x < X; x++)
-			for (unsigned int y = 0; y < Y; y++)
-				for (unsigned int z = 0; z < Z; z++)
-					shift[(z * Z + y) * Y + x] = cexp(2.i * M_PI * ((csh[0] * x) / X + (csh[1] * y) / Y + (csh[2] * z) / Z));
-
-		md_zmul2(DIMS, msk_dims, msk_strs, pattern + size, msk_strs, pattern + size, shift_strs, shift);
-		md_free(shift);
-	}
-
 #if 0
 	float scaling = 1. / estimate_scaling(ksp_dims, NULL, kspace_data);
 #else
-	float scaling = 100. / md_znorm(DIMS, ksp_dims, kspace_data);
+	double scaling = 100. / md_znorm(DIMS, ksp_dims, kspace_data);
+
+	if (1 != ksp_dims[SLICE_DIM]) // SMS
+			scaling *= sqrt(ksp_dims[SLICE_DIM]); 
+
 #endif
 	debug_printf(DP_INFO, "Scaling: %f\n", scaling);
 	md_zsmul(DIMS, ksp_dims, kspace_data, kspace_data, scaling);
@@ -169,21 +166,22 @@ int main_nlinv(int argc, char* argv[])
 	if (normalize) {
 
 		md_zrss(DIMS, ksp_dims, COIL_FLAG, norm, sens);
-		md_zmul2(DIMS, img_dims, img_strs, image, img_strs, image, msk_strs, norm);
+                md_zmul2(DIMS, img_dims, img_strs, image, img_strs, image, img_strs, norm);
 	}
 
-	if (4 == argc) {
+	if (out_sens) {
 
 		long strs[DIMS];
-
 		md_calc_strides(DIMS, strs, ksp_dims, CFL_SIZE);
 
-		if (norm)
+		if (normalize)
 			md_zdiv2(DIMS, ksp_dims, strs, sens, strs, sens, img_strs, norm);
 
 		fftmod(DIMS, ksp_dims, FFT_FLAGS, sens, sens);
 	}
 
+	if (scale_im)
+		md_zsmul(DIMS, img_dims, image, image, 1. / scaling);
 
 	md_free(norm);
 	md_free(mask);
