@@ -1,10 +1,10 @@
 /* Copyright 2013. The Regents of the University of California.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2017. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
- * 2011-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2011-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  *
  *
  * Uecker M, Hohage T, Block KT, Frahm J. Image reconstruction by regularized
@@ -22,7 +22,9 @@
 #include "num/fft.h"
 
 #include "iter/iter3.h"
+#include "iter/iter4.h"
 #include "iter/thresh.h"
+#include "iter/italgos.h"
 
 #include "misc/misc.h"
 #include "misc/types.h"
@@ -30,45 +32,18 @@
 #include "misc/debug.h"
 
 #include "noir/model.h"
+#include "noir/nl.h"
 
 #include "recon.h"
 
 
-struct data {
-
-	struct noir_data* ndata;
-};
-
-
-
-
-static void frw(void* ptr, float* _dst, const float* _src)
-{
-        struct data* data = ptr;
-
-	noir_fun(data->ndata, (complex float*)_dst, (const complex float*)_src);
-}
-
-static void adj(void* ptr, float* _dst, const float* _src)
-{
-        struct data* data = ptr;
-
-	noir_adj(data->ndata, (complex float*)_dst, (const complex float*)_src);
-}
-
-static void der(void* ptr, float* _dst, const float* _src)
-{
-        struct data* data = ptr;
-
-	noir_der(data->ndata, (complex float*)_dst, (const complex float*)_src);
-}
-
 
 const struct noir_conf_s noir_defaults = {
 
 	.iter = 8,
 	.rvc = false,
 	.usegpu = false,
+	.noncart = false,
 	.alpha = 1.,
 	.redu = 2.,
 };
@@ -81,10 +56,12 @@ void noir_recon(const struct noir_conf_s* conf, const long dims[DIMS], complex f
 	long data_dims[DIMS];
 	long img1_dims[DIMS];
 
-	md_select_dims(DIMS, FFT_FLAGS|MAPS_FLAG|CSHIFT_FLAG, imgs_dims, dims);
-	md_select_dims(DIMS, FFT_FLAGS|COIL_FLAG|MAPS_FLAG, coil_dims, dims);
-	md_select_dims(DIMS, FFT_FLAGS|COIL_FLAG, data_dims, dims);
-	md_select_dims(DIMS, FFT_FLAGS, img1_dims, dims);
+	unsigned int fft_flags = FFT_FLAGS|SLICE_FLAG;
+
+	md_select_dims(DIMS, fft_flags|MAPS_FLAG|CSHIFT_FLAG, imgs_dims, dims);
+	md_select_dims(DIMS, fft_flags|COIL_FLAG|MAPS_FLAG, coil_dims, dims);
+	md_select_dims(DIMS, fft_flags|COIL_FLAG, data_dims, dims);
+	md_select_dims(DIMS, fft_flags, img1_dims, dims);
 
 	long skip = md_calc_size(DIMS, imgs_dims);
 	long size = skip + md_calc_size(DIMS, coil_dims);
@@ -92,7 +69,6 @@ void noir_recon(const struct noir_conf_s* conf, const long dims[DIMS], complex f
 
 	long d1[1] = { size };
 	complex float* img = md_alloc_sameplace(1, d1, CFL_SIZE, kspace);
-	complex float* imgH = md_alloc_sameplace(1, d1, CFL_SIZE, kspace);
 
 
 	md_clear(DIMS, imgs_dims, img, CFL_SIZE);
@@ -102,30 +78,39 @@ void noir_recon(const struct noir_conf_s* conf, const long dims[DIMS], complex f
 
 	md_clear(DIMS, coil_dims, img + skip, CFL_SIZE);
 
-	md_clear(DIMS, imgs_dims, imgH, CFL_SIZE);
-	md_clear(DIMS, coil_dims, imgH + skip, CFL_SIZE);
+	struct noir_model_conf_s mconf = noir_model_conf_defaults;
+	mconf.rvc = conf->rvc;
+	mconf.use_gpu = conf->usegpu;
+	mconf.noncart = conf->noncart;
+	mconf.fft_flags = fft_flags;
 
-	struct noir_data* ndata = noir_init(dims, mask, psf, conf->rvc, conf->usegpu);
-	struct data data = { ndata };
+	struct nlop_s* nlop = noir_create(dims, mask, psf, &mconf);
 
-	struct iter3_irgnm_conf irgnm_conf = { .iter = conf->iter, .alpha = conf->alpha, .redu = conf->redu };
-	SET_TYPEID(iter3_irgnm_conf, &irgnm_conf);
+	struct iter3_irgnm_conf irgnm_conf = iter3_irgnm_defaults;
 
-	iter3_irgnm(CAST_UP(&irgnm_conf), frw, der, adj, &data, size * 2, (float*)img, NULL,
-				data_size * 2, (const float*)kspace);
+	irgnm_conf.iter = conf->iter;
+	irgnm_conf.alpha = conf->alpha;
+	irgnm_conf.redu = conf->redu;
+	irgnm_conf.cgtol = 0.1f;
+	irgnm_conf.nlinv_legacy = true;
+
+	iter4_irgnm(CAST_UP(&irgnm_conf),
+			nlop,
+			size * 2, (float*)img, NULL,
+			data_size * 2, (const float*)kspace);
 
 	md_copy(DIMS, imgs_dims, outbuf, img, CFL_SIZE);
 
 	if (NULL != sensout) {
 
 		assert(!conf->usegpu);
-		noir_forw_coils(ndata, sensout, img + skip);
+		noir_forw_coils(noir_get_data(nlop), sensout, img + skip);
+		fftmod(DIMS, coil_dims, fft_flags, sensout, sensout);
 	}
 
-	noir_free(ndata);
+	nlop_free(nlop);
 
 	md_free(img);
-	md_free(imgH);
 }
 
 
