@@ -1,11 +1,11 @@
 /* Copyright 2014-2015. The Regents of the University of California.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2017. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2014-2016 Frank Ong <frankong@berkeley.edu>
- * 2014-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2014-2017 Frank Ong <frankong@berkeley.edu>
+ * 2014-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  *
  * Strang G. A proposal for Toeplitz matrix calculations. Journal Studies in Applied Math. 1986; 74(2):171-17
  *
@@ -40,7 +40,7 @@
 
 struct nufft_conf_s nufft_conf_defaults = {
 
-	.toeplitz = false,
+	.toeplitz = true,
 	.pcycle = false,
 };
 
@@ -97,8 +97,8 @@ struct nufft_data {
 	long* psf_strs;
 	long* wgh_strs;
 
-	const struct linop_s* rfft_op;   ///< Pcycle FFT operator
-	unsigned int rand_state;
+	const struct linop_s* cfft_op;   ///< Pcycle FFT operator
+	unsigned int cycle;
 	long* rcml_dims;		///< Pcycle Coil + linear phase dimension
 	long* rlph_dims;		///< Pcycle Linear phase dimension
 	long* rpsf_dims;		///< Pcycle Point spread function dimension
@@ -109,7 +109,7 @@ struct nufft_data {
 	
 };
 
-DEF_TYPEID(nufft_data);
+static DEF_TYPEID(nufft_data);
 
 
 static void nufft_free_data(const linop_data_t* data);
@@ -293,8 +293,8 @@ struct linop_s* nufft_create(unsigned int N,			///< Number of dimension
 	if (conf.pcycle) {
 
 		debug_printf(DP_DEBUG1, "NUFFT: Pcycle Mode\n");
-		data->rand_state = 0;
-		data->rfft_op = linop_fft_create(N, data->cim_dims, FFT_FLAGS);
+		data->cycle = 0;
+		data->cfft_op = linop_fft_create(N, data->cim_dims, FFT_FLAGS);
 	}
 
 
@@ -360,7 +360,7 @@ struct nufft_precond_data {
 };
 
 
-DEF_TYPEID(nufft_precond_data);
+static DEF_TYPEID(nufft_precond_data);
 
 
 static void nufft_precond_apply(const operator_data_t* _data, unsigned int M, void* args[M])
@@ -393,7 +393,7 @@ static void nufft_precond_del(const operator_data_t* _data)
 
 const struct operator_s* nufft_precond_create(const struct linop_s* nufft_op)
 {
-	const struct nufft_data* data = linop_get_data(nufft_op);
+	const struct nufft_data* data = CAST_DOWN(nufft_data, linop_get_data(nufft_op));
 
 	PTR_ALLOC(struct nufft_precond_data, pdata);
 	SET_TYPEID(nufft_precond_data, pdata);
@@ -401,7 +401,7 @@ const struct operator_s* nufft_precond_create(const struct linop_s* nufft_op)
 	assert(data->conf.toeplitz);
 
 	unsigned int N = data->N;
-	unsigned int ND = pdata->N + 3;
+	unsigned int ND = N + 3;
 
 	pdata->N = N;
 	pdata->cim_dims = *TYPE_ALLOC(long[ND]);
@@ -479,7 +479,10 @@ complex float* compute_psf(unsigned int N, const long img2_dims[N], const long t
 	long ksp_dims1[N];
 	md_select_dims(N, ~MD_BIT(0), ksp_dims1, trj_dims);
 
-	struct linop_s* op2 = nufft_create(N, ksp_dims1, img2_dims, trj_dims, traj, NULL, nufft_conf_defaults);
+	struct nufft_conf_s conf = nufft_conf_defaults;
+	conf.toeplitz = false;	// avoid infinite loop
+
+	struct linop_s* op2 = nufft_create(N, ksp_dims1, img2_dims, trj_dims, traj, NULL, conf);
 
 	complex float* ones = md_alloc(N, ksp_dims1, CFL_SIZE);
 	md_zfill(N, ksp_dims1, ones, 1.);
@@ -577,12 +580,22 @@ static void nufft_free_data(const linop_data_t* _data)
 	free(data->lph_strs);
 	free(data->psf_strs);
 	free(data->wgh_strs);
-	
+
+	free(data->rlph_dims);
+	free(data->rpsf_dims);
+	free(data->rcml_dims);
+	free(data->rlph_strs);
+	free(data->rpsf_strs);
+	free(data->rcml_strs);
+
+	free(data->cm2_dims);
+
 	md_free(data->grid);
 	md_free(data->linphase);
 	md_free(data->psf);
 	md_free(data->fftmod);
 	md_free(data->weights);
+	md_free(data->roll);
 
 #ifdef USE_CUDA
 	md_free(data->linphase_gpu);
@@ -591,7 +604,7 @@ static void nufft_free_data(const linop_data_t* _data)
 #endif
 	linop_free(data->fft_op);
 	if (data->conf.pcycle)
-		linop_free(data->rfft_op);
+		linop_free(data->cfft_op);
 
 	free(data);
 }
@@ -741,42 +754,22 @@ static void toeplitz_mult(const struct nufft_data* data, complex float* dst, con
 	md_zfmacc2(ND, data->cml_dims, data->cim_strs, dst, data->cml_strs, grid, data->lph_strs, linphase);
 }
 
-static unsigned int rand_lim(unsigned int* state, unsigned int limit)
-{
-        /* int divisor = RAND_MAX / (limit + 1); */
-        /* unsigned int retval; */
-
-        /* do { */
-        /*         retval = rand_r(state) / divisor; */
-
-        /* } while (retval > limit); */
-	unsigned int retval = *state % (limit + 1);
-	state[0]++;
-
-        return retval;
-}
-
-
 static void toeplitz_mult_pcycle(const struct nufft_data* data, complex float* dst, const complex float* src)
 {
-	unsigned int nshifts = data->lph_dims[data->N];
-        unsigned int r = rand_lim((unsigned int*) &(data->rand_state), nshifts-1);
+	unsigned int ncycles = data->lph_dims[data->N];
+        ((struct nufft_data*) data)->cycle = (data->cycle + 1) % ncycles;
 	
-	const complex float* rlinphase = data->linphase + r * md_calc_size(data->N, data->lph_dims);
-	const complex float* rpsf = data->psf + r * md_calc_size(data->N, data->psf_dims);
+	const complex float* clinphase = data->linphase + data->cycle * md_calc_size(data->N, data->lph_dims);
+	const complex float* cpsf = data->psf + data->cycle * md_calc_size(data->N, data->psf_dims);
 	complex float* grid = data->grid;
 
-#ifdef USE_CUDA
-	printf("Not Implemented.\n");
-	exit(1);
-#endif
-	md_zmul2(data->N, data->cim_dims, data->cim_strs, grid, data->cim_strs, src, data->img_strs, rlinphase);
+	md_zmul2(data->N, data->cim_dims, data->cim_strs, grid, data->cim_strs, src, data->img_strs, clinphase);
 
-	linop_forward(data->rfft_op, data->N, data->cim_dims, grid, data->N, data->cim_dims, grid);
-	md_zmul2(data->N, data->cim_dims, data->cim_strs, grid, data->cim_strs, grid, data->img_strs, rpsf);
-	linop_adjoint(data->rfft_op, data->N, data->cim_dims, grid, data->N, data->cim_dims, grid);
+	linop_forward(data->cfft_op, data->N, data->cim_dims, grid, data->N, data->cim_dims, grid);
+	md_zmul2(data->N, data->cim_dims, data->cim_strs, grid, data->cim_strs, grid, data->img_strs, cpsf);
+	linop_adjoint(data->cfft_op, data->N, data->cim_dims, grid, data->N, data->cim_dims, grid);
 
-	md_zmulc2(data->N, data->cim_dims, data->cim_strs, dst, data->cim_strs, grid, data->img_strs, rlinphase);
+	md_zmulc2(data->N, data->cim_dims, data->cim_strs, dst, data->cim_strs, grid, data->img_strs, clinphase);
 }
 
 
