@@ -38,7 +38,7 @@ int main_normalize(int argc, char* argv[])
 {
 	bool l1 = false;
 
-	l1 = mini_cmdline_bool(argc, argv, 'b', 3, usage_str, help_str);
+	l1 = mini_cmdline_bool(&argc, argv, 'b', 3, usage_str, help_str);
 
 	num_init();
 
