@@ -44,8 +44,7 @@ void overlapandadd(int N, const long dims[N], const long blk[N], complex float*
 		ndim3[i * 2 + 0] = blk[i];
 	}
 
-	long T = md_calc_size(2 * N, L);
-	complex float* tmp = xmalloc(T * 8);
+	complex float* tmp = md_alloc(2 * N, L, CFL_SIZE);
 
 //	conv_causal_extend(2 * N, L, tmp, ndims, src1, ndim2, src2);
 	conv(2 * N, ~0, CONV_EXTENDED, CONV_CAUSAL, L, tmp, ndims, src1, ndim2, src2);
@@ -62,7 +61,7 @@ void overlapandadd(int N, const long dims[N], const long blk[N], complex float*
 	md_clear(2 * N, ndim3, dst, CFL_SIZE);
 	md_zadd2(2 * N, L, str3, dst, str3, dst, str2, tmp);
 	
-	free(tmp);	
+	md_free(tmp);
 }
 
 
@@ -95,8 +94,7 @@ void overlapandsave(int N, const long dims[N], const long blk[N], complex float*
 		ndim3[i * 2 + 0] = blk[i];
 	}
 
-	long T = md_calc_size(2 * N, L);
-	complex float* tmp = xmalloc(T * 8);
+	complex float* tmp = md_alloc(2 * N, L, CFL_SIZE);
 
 	long str1[2 * N];
 	long str2[2 * N];
@@ -110,7 +108,7 @@ void overlapandsave(int N, const long dims[N], const long blk[N], complex float*
 	md_copy2(2 * N, ndim3, str2, tmp, str1, src1, 8);
 	conv(2 * N, ~0, CONV_VALID, CONV_CAUSAL, ndims, dst, L, tmp, ndim2, src2);
 
-	free(tmp);	
+	md_free(tmp);
 }
 
 
@@ -437,13 +435,9 @@ void overlapandsave2NE(int N, unsigned int flags, const long blk[N], const long
 		}
 	}
 
-	long R = md_calc_size(N, odims);
-	long T = md_calc_size(2 * N, tdims);
-	long S = md_calc_size(N, dims1B);
-
-	complex float* src1B = xmalloc(S * sizeof(complex float));
-	complex float* tmp = xmalloc(T * sizeof(complex float));
-	complex float* tmpX = xmalloc(R * sizeof(complex float));
+	complex float* src1B = md_alloc(N, dims1B, CFL_SIZE);
+	complex float* tmp = md_alloc(2 * N, tdims, CFL_SIZE);
+	complex float* tmpX = md_alloc(N, odims, CFL_SIZE);
 
 	long str1[2 * N];
 	long str2[2 * N];
@@ -478,16 +472,16 @@ void overlapandsave2NE(int N, unsigned int flags, const long blk[N], const long
 
 	md_resize_center(N, dims1, dst, dims1B, src1B, sizeof(complex float));
 
-	free(src1B);
-	free(tmpX);
-	free(tmp);
+	md_free(src1B);
+	md_free(tmpX);
+	md_free(tmp);
 }
 
 
 
 
 
-void overlapandsave2NEB(const struct vec_ops* ops, int N, unsigned int flags, const long blk[N], const long odims[N], complex float* dst, const long dims1[N], const complex float* src1, const long dims2[N], const complex float* src2, const long mdims[N], const complex float* msk)
+void overlapandsave2NEB(int N, unsigned int flags, const long blk[N], const long odims[N], complex float* dst, const long dims1[N], const complex float* src1, const long dims2[N], const complex float* src2, const long mdims[N], const complex float* msk)
 {
 	long dims1B[N];
 
@@ -618,10 +612,6 @@ void overlapandsave2NEB(const struct vec_ops* ops, int N, unsigned int flags, co
 	assert(tdims[N + 1] == nodims[N + 1]);
 	assert(tdims[N + 2] == nodims[N + 2]);
 
-	long R = md_calc_size(N, nodims);
-	long T = md_calc_size(N, tdims);
-
-
 	//complex float* src1C = xmalloc(S * sizeof(complex float));
 	complex float* src1C = dst;
 
@@ -632,8 +622,8 @@ void overlapandsave2NEB(const struct vec_ops* ops, int N, unsigned int flags, co
 	for (int j = 0; j < nodims[N + 1]; j++) {
 	for (int i = 0; i < nodims[N + 0]; i++) {
 
-		complex float* tmp = (complex float*)ops->allocate(2 * T);
-		complex float* tmpX = (complex float*)ops->allocate(2 * R);
+		complex float* tmp = md_alloc_sameplace(N, tdims, CFL_SIZE, dst);
+		complex float* tmpX = md_alloc_sameplace(N, nodims, CFL_SIZE, dst);
 
 		long off1 = str1[N + 0] * i + str1[N + 1] * j + str1[N + 2] * k;
 		long off2 = mstr[N + 0] * i + mstr[N + 1] * j + mstr[N + 2] * k;
@@ -646,8 +636,8 @@ void overlapandsave2NEB(const struct vec_ops* ops, int N, unsigned int flags, co
 		#pragma omp critical
 		md_zadd2(N, tdims, str1, ((void*)src1C) + off1, str1, ((void*)src1C) + off1, str2,  tmp);
 
-		ops->del((void*)tmpX);
-		ops->del((void*)tmp);
+		md_free(tmpX);
+		md_free(tmp);
 	}}}
 
 	//md_resizec(N, dims1, dst, dims1B, src1C, sizeof(complex float));
@@ -659,7 +649,7 @@ void overlapandsave2NEB(const struct vec_ops* ops, int N, unsigned int flags, co
 
 
 
-void overlapandsave2HB(const struct vec_ops* ops, int N, unsigned int flags, const long blk[N], const long dims1[N], complex float* dst, const long odims[N], const complex float* src1, const long dims2[N], const complex float* src2, const long mdims[N], const complex float* msk)
+void overlapandsave2HB(int N, unsigned int flags, const long blk[N], const long dims1[N], complex float* dst, const long odims[N], const complex float* src1, const long dims2[N], const complex float* src2, const long mdims[N], const complex float* msk)
 {
 	long dims1B[N];
 
@@ -786,22 +776,19 @@ void overlapandsave2HB(const struct vec_ops* ops, int N, unsigned int flags, con
 	assert(tdims[N + 1] == nodims[N + 1]);
 	assert(tdims[N + 2] == nodims[N + 2]);
 
-	long R = md_calc_size(N, nodims);
-	long T = md_calc_size(N, tdims);
-
 
 	//complex float* src1C = xmalloc(S * sizeof(complex float));
 	complex float* src1C = dst;
 
-	md_clear(N, dims1B, src1C, sizeof(complex float));	// must be done here
+	md_clear(N, dims1B, src1C, CFL_SIZE);	// must be done here
 
 	#pragma omp parallel for collapse(3)
 	for (int k = 0; k < nodims[N + 2]; k++) {
 	for (int j = 0; j < nodims[N + 1]; j++) {
 	for (int i = 0; i < nodims[N + 0]; i++) {
 
-		    complex float* tmp = (complex float*)ops->allocate(2 * T);
-		    complex float* tmpX = (complex float*)ops->allocate(2 * R);
+		    complex float* tmp = md_alloc_sameplace(N, tdims, CFL_SIZE, dst);
+		    complex float* tmpX = md_alloc_sameplace(N, nodims, CFL_SIZE, dst);
 
 		    long off1 = str1[N + 0] * i + str1[N + 1] * j + str1[N + 2] * k;
 		    long off2 = mstr[N + 0] * i + mstr[N + 1] * j + mstr[N + 2] * k;
@@ -813,8 +800,8 @@ void overlapandsave2HB(const struct vec_ops* ops, int N, unsigned int flags, con
 		    #pragma omp critical
 		    md_zadd2(N, tdims, str1, ((void*)src1C) + off1, str1, ((void*)src1C) + off1, str2,  tmp);
 
-		    ops->del((void*)tmpX);
-		    ops->del((void*)tmp);
+		    md_free(tmpX);
+		    md_free(tmp);
 	}}}
 }
 
