@@ -1,14 +1,16 @@
 /* Copyright 2013-2015 The Regents of the University of California.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2017. Martin Uecker.
+ * Copyright 2017. University of Oxford.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2012-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2013 Dara Bahri <dbahri123@gmail.com>
  * 2014 Frank Ong <frankong@berkeley.edu>
  * 2014-2015 Jonathan Tamir <jtamir@eecs.berkeley.edu>
  * 2016 Siddharth Iyer <sid8795@gmail.com>
+ * 2017 Sofia Dimoudi <sofia.dimoudi@cardiov.ox.ac.uk>
  *
  *
  * Operations on arrays of complex single-precision floating
@@ -35,12 +37,12 @@
 #include "num/flpmath.h"
 #include "num/vecops.h"
 #include "num/optimize.h"
+#include "num/blas.h"
 
 #include "misc/misc.h"
 #include "misc/types.h"
 #include "misc/debug.h"
 
-
 #ifdef USE_CUDA
 #include "num/gpuops.h"
 /*
@@ -326,6 +328,7 @@ static void nary_z2opd(struct nary_opt_data_s* data, void* ptr[])
 	(*(z2opd_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
 }
 
+__attribute__((unused))
 static void make_z2opd(size_t offset, unsigned int D, const long dim[D], const long ostr[D], complex double* optr, const long istr1[D], const complex float* iptr1)
 {
 	size_t sizes[2] = { sizeof(complex double), sizeof(complex float) };
@@ -333,7 +336,6 @@ static void make_z2opd(size_t offset, unsigned int D, const long dim[D], const l
 	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, sizes, nary_z2opd, &offset);
 }
 
-void* unsued = make_z2opd;
 
 static void nary_2opd(struct nary_opt_data_s* data, void* ptr[])
 {
@@ -994,43 +996,158 @@ void md_zfloat2double(unsigned int D, const long dims[D], complex double* dst, c
 }
 
 
-
-/**
- * Store max dimensions of dims1 and dims2 and check for basic consistency
- * FIXME move to some other place?
+/*
+ * A A A ok
+ * A A 1 ok
+ * A 1 A ok
+ * 1 A A ok
+ * A 1 1 !
+ * 1 A 1 !
+ * 1 1 A !
+ * 1 1 1 ok
  */
-static void md_mmdims(unsigned int D, long max_dims[D], const long dims1[D], const long dims2[D])
+void md_tenmul_dims(unsigned int D, long max_dims[D], const long out_dims[D], const long in1_dims[D], const long in2_dims[D])
 {
-	for (unsigned int i = 0; i < D; i++) {
+	md_max_dims(D, ~0u, max_dims, in1_dims, out_dims);
 
-		if ((dims1[i] > 1) && (dims2[i] == 1)) {
+	long max2_dims[D];
+	md_max_dims(D, ~0u, max2_dims, in2_dims, out_dims);
 
-			max_dims[i] = dims1[i];
-
-		} else
-		if ((dims1[i] == 1) && (dims2[i] > 1)) {
-
-			max_dims[i] = dims2[i];
-
-		} else {
-
-			assert(dims1[i] == dims2[i]);
-			max_dims[i] = dims1[i];
-		}
-	}
+	assert(md_check_compat(D, 0u, max_dims, max2_dims));
 }
 
+
+static bool detect_matrix(const long dims[3], const long ostrs[3], const long mstrs[3], const long istrs[3])
+{
+        return (   (0 == ostrs[1])
+                && (0 == mstrs[2])
+                && (0 == istrs[0])
+                && ((CFL_SIZE == ostrs[0]) && (ostrs[0] * dims[0] == ostrs[2]))
+                && ((CFL_SIZE == mstrs[0]) && (mstrs[0] * dims[0] == mstrs[1]))
+                && ((CFL_SIZE == istrs[1]) && (istrs[1] * dims[1] == istrs[2])));
+}
+
+
+static bool simple_matmul(unsigned int N, const long max_dims[N], const long ostrs[N], complex float* out,
+		const long mstrs[N], const complex float* mat, const long istrs[N], const complex float* in)
+{
+	long dims[N];
+	md_copy_dims(N, dims, max_dims);
+
+	long ostrs2[N];
+	md_copy_strides(N, ostrs2, ostrs);
+
+	long mstrs2[N];
+	md_copy_strides(N, mstrs2, mstrs);
+
+	long istrs2[N];
+	md_copy_strides(N, istrs2, istrs);
+
+	long (*strs[3])[N] = { &ostrs2, &istrs2, &mstrs2 };
+	unsigned int ND = simplify_dims(3, N, dims, strs);
+
+	long C = dims[0];
+	long B = dims[1];
+	long A = dims[2];
+
+	if ((3 == ND) && detect_matrix(dims, ostrs2, istrs2, mstrs2)) {
+
+		debug_printf(DP_DEBUG4, "matmul: matrix multiplication (1).\n");
+#if 0
+		// num/linalg.h
+
+		mat_mul(A, B, C,
+			*(complex float (*)[A][C])out,
+			*(const complex float (*)[A][B])mat,
+			*(const complex float (*)[B][C])in);
+#else
+		blas_matrix_multiply(C, A, B,
+			*(complex float (*)[A][C])out,
+			*(const complex float (*)[B][C])in,
+			*(const complex float (*)[A][B])mat);
+#endif
+		return true;
+	}
+
+	if ((3 == ND) && detect_matrix(dims, ostrs2, mstrs2, istrs2)) {
+
+		debug_printf(DP_DEBUG4, "matmul: matrix multiplication (2).\n");
+#if 0
+		// num/linalg.h
+
+		mat_mul(A, B, C,
+			*(complex float (*)[A][C])out,
+			*(const complex float (*)[A][B])in,
+			*(const complex float (*)[B][C])mat);
+#else
+		blas_matrix_multiply(C, A, B,
+			*(complex float (*)[A][C])out,
+			*(const complex float (*)[B][C])mat,
+			*(const complex float (*)[A][B])in);
+#endif
+		return true;
+	}
+
+
+	return false;
+}
+
+
+/*
+ * tenmul (tensor multiplication) family of functions are revised
+ * versions of the matmul functions.
+ */
+void md_ztenmul2(unsigned int D, const long max_dims[D], const long out_strs[D], complex float* out, const long in1_strs[D], const complex float* in1, const long in2_strs[D], const complex float* in2)
+{
+	if (simple_matmul(D, max_dims, out_strs, out, in2_strs, in2, in1_strs, in1))
+		return;
+
+	md_clear2(D, max_dims, out_strs, out, CFL_SIZE);
+	md_zfmac2(D, max_dims, out_strs, out, in1_strs, in1, in2_strs, in2);
+}
+
+
+void md_ztenmulc2(unsigned int D, const long max_dims[D], const long out_strs[D], complex float* out, const long in1_strs[D], const complex float* in1, const long in2_strs[D], const complex float* in2)
+{
+	md_clear2(D, max_dims, out_strs, out, CFL_SIZE);
+	md_zfmacc2(D, max_dims, out_strs, out, in1_strs, in1, in2_strs, in2);
+}
+
+
+void md_ztenmul(unsigned int D, const long out_dims[D], complex float* out, const long in1_dims[D], const complex float* in1, const long in2_dims[D], const complex float* in2)
+{
+	long max_dims[D];
+	md_tenmul_dims(D, max_dims, out_dims, in1_dims, in2_dims);
+
+	md_ztenmul2(D, max_dims, MD_STRIDES(D, out_dims, CFL_SIZE), out,
+				 MD_STRIDES(D, in1_dims, CFL_SIZE), in1,
+				 MD_STRIDES(D, in2_dims, CFL_SIZE), in2);
+}
+
+
+void md_ztenmulc(unsigned int D, const long out_dims[D], complex float* out, const long in1_dims[D], const complex float* in1, const long in2_dims[D], const complex float* in2)
+{
+	long max_dims[D];
+	md_tenmul_dims(D, max_dims, out_dims, in1_dims, in2_dims);
+
+	md_ztenmulc2(D, max_dims, MD_STRIDES(D, out_dims, CFL_SIZE), out,
+				  MD_STRIDES(D, in1_dims, CFL_SIZE), in1,
+				  MD_STRIDES(D, in2_dims, CFL_SIZE), in2);
+}
+
+
+
+/*
+ * matmul family of functions is deprecated - use tenmul instead
+ */
+
 static void md_zmatmul2_priv(unsigned int D, const long out_dims[D], const long out_strs[D], complex float* dst, const long mat_dims[D], const long mat_strs[D], const complex float* mat, const long in_dims[D], const long in_strs[D], const complex float* src, bool conj)
 {
-	UNUSED(mat_dims);
-
 	long max_dims[D];
-	md_mmdims(D, max_dims, in_dims, out_dims);
+	md_tenmul_dims(D, max_dims, out_dims, mat_dims, in_dims);
 
-	long max2_dims[D];
-	md_mmdims(D, max2_dims, mat_dims, out_dims);
-
-	assert(md_check_compat(D, 0, max_dims, max2_dims));
+	if ((!conj) && simple_matmul(D, max_dims, out_strs, dst, mat_strs, mat, in_strs, src))
+		return;
 
 	md_clear2(D, out_dims, out_strs, dst, CFL_SIZE);
 	(conj ? md_zfmacc2 : md_zfmac2)(D, max_dims, out_strs, dst, in_strs, src, mat_strs, mat);
@@ -1062,6 +1179,7 @@ void md_zmatmulc(unsigned int D, const long out_dims[D], complex float* dst, con
 /**
  * Matrix multiplication (with strides)
  * FIXME simplify interface?
+ * FIXME: implementation assumes strides == 0 for dims == 1
  */
 void md_zmatmul2(unsigned int D, const long out_dims[D], const long out_strs[D], complex float* dst, const long mat_dims[D], const long mat_strs[D], const complex float* mat, const long in_dims[D], const long in_strs[D], const complex float* src)
 {
@@ -1845,8 +1963,10 @@ float md_scalar2(unsigned int D, const long dim[D], const long str1[D], const fl
 	md_fmacD2(D, dim, stro, retp, str1, ptr1, str2, ptr2);
 
 #ifdef USE_CUDA
-	if (cuda_ondevice(ptr1))
+	if (cuda_ondevice(ptr1)) {
 		md_copy(1, (long[1]){ 1 }, &ret, retp, DL_SIZE);
+		md_free(retp);
+	}
 #endif
 	return ret;
 }
@@ -1993,8 +2113,10 @@ complex float md_zscalar2(unsigned int D, const long dim[D], const long str1[D],
 	md_zfmaccD2(D, dim, stro, retp, str1, ptr1, str2, ptr2);
 
 #ifdef USE_CUDA
-	if (cuda_ondevice(ptr1))
+	if (cuda_ondevice(ptr1)) {
 		md_copy(1, (long[1]){ 1 }, &ret, retp, CDL_SIZE);
+		md_free(retp);
+	}
 #endif
 
 	return (complex float)ret;
@@ -2182,8 +2304,10 @@ float md_asum2(unsigned int D, const long dims[D], const long strs[D], const flo
 	md_axpy2(D, dims, MD_STRIDES(D, dims0, FL_SIZE), retp, 1., strs1, tmp);
 
 #ifdef USE_CUDA
-	if (cuda_ondevice(ptr))
+	if (cuda_ondevice(ptr)) {
 		md_copy(D, dims0, &ret, retp, FL_SIZE);
+		md_free(retp);
+	}
 #endif
 
 	md_free(tmp);
@@ -2290,14 +2414,13 @@ void md_rss(unsigned int D, const long dims[D], unsigned int flags, float* dst,
 
 
 /**
- * Root of sum of squares along selected dimensions
+ * Sum of squares along selected dimensions
  *
  * @param dims -- full dimensions of src image
  * @param flags -- bitmask for applying the root of sum of squares, i.e. the dimensions that will not stay
  */
-void md_zrss(unsigned int D, const long dims[D], unsigned int flags, complex float* dst, const complex float* src)
+void md_zss(unsigned int D, const long dims[D], unsigned int flags, complex float* dst, const complex float* src)
 {
-#if 1
 	long str1[D];
 	long str2[D];
 	long dims2[D];
@@ -2309,6 +2432,22 @@ void md_zrss(unsigned int D, const long dims[D], unsigned int flags, complex flo
 
 	md_clear(D, dims2, dst, CFL_SIZE);
 	md_zfmacc2(D, dims, str2, dst, str1, src, str1, src);
+}
+
+
+
+/**
+ * Root of sum of squares along selected dimensions
+ *
+ * @param dims -- full dimensions of src image
+ * @param flags -- bitmask for applying the root of sum of squares, i.e. the dimensions that will not stay
+ */
+void md_zrss(unsigned int D, const long dims[D], unsigned int flags, complex float* dst, const complex float* src)
+{
+	long dims2[D];
+	md_select_dims(D, ~flags, dims2, dims);
+#if 1
+	md_zss(D, dims, flags, dst, src);
 
 #if 1
 	long dims2R[D + 1];
@@ -2327,6 +2466,96 @@ void md_zrss(unsigned int D, const long dims[D], unsigned int flags, complex flo
 
 
 
+/**
+ * Compute variance or standard deviation along selected dimensions (with strides)
+ *
+ * @param dims -- full dimensions of src image
+ * @param flags -- bitmask for calculating var/std, i.e. the dimensions that will not stay
+ * @param variance -- true if computing variance, false if computing standard deviation
+ */
+static void md_zvarstd2(unsigned int D, const long dims[D], unsigned int flags, const long ostr[D], complex float* optr, const long istr[D], const complex float* iptr, bool variance)
+{
+	long odims[D];
+	long fdims[D];
+
+	md_select_dims(D, ~flags, odims, dims);
+	md_select_dims(D, flags, fdims, dims);
+
+	complex float* tmp = md_alloc_sameplace(D, dims, CFL_SIZE, iptr);
+
+	md_zavg2(D, dims, flags, ostr, optr, istr, iptr);
+	md_zsub2(D, dims, istr, tmp, istr, iptr, ostr, optr);
+
+	double scale = variance ? md_calc_size(D, fdims) - 1. : sqrtf(md_calc_size(D, fdims) - 1.);
+	(variance ? md_zss : md_zrss)(D, dims, flags, optr, tmp);
+
+	md_zsmul2(D, odims, ostr, optr, ostr, optr, 1. / scale);
+
+	md_free(tmp);
+}
+
+
+
+/**
+ * Compute variance along selected dimensions (without strides)
+ *
+ * @param dims -- full dimensions of src image
+ * @param flags -- bitmask for calculating variance, i.e. the dimensions that will not stay
+ */
+void md_zvar(unsigned int D, const long dims[D], unsigned int flags, complex float* optr, const complex float* iptr)
+{
+	long odims[D];
+	md_select_dims(D, ~flags, odims, dims);
+
+	md_zvar2(D, dims, flags,
+			MD_STRIDES(D, odims, CFL_SIZE), optr,
+			MD_STRIDES(D, dims, CFL_SIZE), iptr);
+}
+
+
+/**
+ * Compute variance along selected dimensions (with strides)
+ *
+ * @param dims -- full dimensions of src image
+ * @param flags -- bitmask for calculating variance, i.e. the dimensions that will not stay
+ */
+void md_zvar2(unsigned int D, const long dims[D], unsigned int flags, const long ostr[D], complex float* optr, const long istr[D], const complex float* iptr)
+{
+	md_zvarstd2(D, dims, flags, ostr, optr, istr, iptr, true);
+}
+
+
+
+/**
+ * Compute standard deviation along selected dimensions (without strides)
+ *
+ * @param dims -- full dimensions of src image
+ * @param flags -- bitmask for calculating standard deviation, i.e. the dimensions that will not stay
+ */
+void md_zstd(unsigned int D, const long dims[D], unsigned int flags, complex float* optr, const complex float* iptr)
+{
+	long odims[D];
+	md_select_dims(D, ~flags, odims, dims);
+
+	md_zvarstd2(D, dims, flags,
+			MD_STRIDES(D, odims, CFL_SIZE), optr,
+			MD_STRIDES(D, dims, CFL_SIZE), iptr, false);
+}
+
+
+/**
+ * Compute standard deviation along selected dimensions (with strides)
+ *
+ * @param dims -- full dimensions of src image
+ * @param flags -- bitmask for calculating standard deviation, i.e. the dimensions that will not stay
+ */
+void md_zstd2(unsigned int D, const long dims[D], unsigned int flags, const long ostr[D], complex float* optr, const long istr[D], const complex float* iptr)
+{
+	md_zvarstd2(D, dims, flags, ostr, optr, istr, iptr, false);
+}
+
+
+
 /**
  * Average along flagged dimensions (without strides)
  *
@@ -2684,8 +2913,6 @@ void md_zsoftthresh2(unsigned int D, const long dims[D], float lambda, unsigned
 	md_free(tmp_norm);
 }
 
-
-
 /**
  * Soft thresholding using norm along arbitrary dimension (without strides)
  *
@@ -2708,7 +2935,170 @@ void md_zsoftthresh(unsigned int D, const long dims[D], float lambda, unsigned i
 	md_zsoftthresh2(D, dims, lambda, flags, strs, optr, strs, iptr);
 }
 
+/**
+ * Hard Thresholding mask complex array (nonzero support of k-largest elements)
+ * Writes to the output a mask of the non-zero elements in the input
+ *
+ * return HardThreshSupp(ptr)
+ */
+static void nary_zhardthresh_mask(struct nary_opt_data_s* data, void* ptr[])
+{
+	data->ops->zhardthresh_mask(data->size, (*(unsigned int*)data->data_ptr), ptr[0], ptr[1]);
+}
 
+/**
+ * Produces a mask (1s and 0s) of the non-zero support of a hard thresholded input vector
+ * Multi-dimensional operation with strides
+ * Hard thresholding is performed by selection of the k largest elements in input.
+ *
+ * @param D number of dimensions
+ * @param dim dimensions of input/output
+ * @param k threshold parameter
+ * @param flags flags for joint operation
+ * @param ostr output strides
+ * @param optr pointer to output
+ * @param istr input strides
+ * @param iptr pointer to input
+ */
+void md_zhardthresh_mask2(unsigned int D, const long dim[D], unsigned int k, unsigned int flags, complex float* tmp_norm, const long ostr[D], complex float* optr, const long istr[D], const complex float* iptr)
+{
+	if (0 == flags) {
+		optimized_twoop_oi(D, dim, ostr, optr, istr, iptr, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh_mask, &k);
+		return;
+	}
+
+	long norm_dims[D];
+	long norm_strs[D];
+
+	md_select_dims(D, ~flags, norm_dims, dim);
+	md_calc_strides(D, norm_strs, norm_dims, CFL_SIZE);
+
+	md_zrss(D, dim, flags, tmp_norm, iptr);
+	optimized_twoop_oi(D, norm_dims, norm_strs, tmp_norm, norm_strs, tmp_norm, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh_mask, &k);
+	md_copy2(D, dim, ostr, optr, norm_strs, tmp_norm, CFL_SIZE);
+}
+
+/**
+ * Produces a mask (1s and 0s) of the non-zero support of a hard thresholded input vector
+ * Multi-dimensional operation using the same strides for input and output.
+ * Hard thresholding is performed by selection of the k largest elements in input.
+ *
+ * @param D number of dimensions
+ * @param dim dimensions of input/output
+ * @param k threshold parameter
+ * @param optr pointer to output
+ * @param iptr pointer to input
+ */
+void md_zhardthresh_mask(unsigned int D, const long dim[D], unsigned int k, unsigned int flags, complex float* optr, const complex float* iptr)
+{
+	long str[D];
+	md_calc_strides(D, str, dim, CFL_SIZE);
+
+	long norm_dims[D];
+	md_select_dims(D, ~flags, norm_dims, dim);
+
+	complex float* tmp_norm = md_alloc_sameplace(D, norm_dims, CFL_SIZE, iptr);
+
+	md_zhardthresh_mask2(D, dim, k, flags, tmp_norm, str, optr, str, iptr);
+
+	md_free(tmp_norm);
+
+}
+
+/**
+ * Joint Hard thresholding  (with strides)
+ * Performs hard thresholding to the norm along dimension specified by flags
+ * Applies the support of thresholded norm to every vector along that dimension
+ * Hard thresholding refers to the selection of the k largest elements in vector.
+ *
+ * @param D number of dimensions
+ * @param dims dimensions of input/output
+ * @param k threshold (sorted) index
+ * @param flags jointly thresholded dimensions
+ * @param tmp_norm temporary array for joint operation
+ * @param ostrs destination strides
+ * @param optr destination -- thresholded values
+ * @param istrs source strides
+ * @param iptr source -- values to be thresholded
+ */
+void md_zhardthresh_joint2(unsigned int D, const long dims[D], unsigned int k, unsigned int flags, complex float* tmp_norm, const long ostrs[D], complex float* optr, const long istrs[D], const complex float* iptr)
+{
+	long norm_dims[D];
+	long norm_strs[D];
+
+	md_select_dims(D, ~flags, norm_dims, dims);
+	md_calc_strides(D, norm_strs, norm_dims, CFL_SIZE);
+
+	md_zrss(D, dims, flags, tmp_norm, iptr);
+	optimized_twoop_oi(D, norm_dims, norm_strs, tmp_norm, norm_strs, tmp_norm, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh_mask, &k);
+	md_zmul2(D, dims, ostrs, optr, norm_strs, tmp_norm, istrs, iptr);
+}
+
+/**
+ * Hard Thresholding complex array (select k-largest elements)
+ *
+ * return HardThresh(ptr)
+ */
+static void nary_zhardthresh(struct nary_opt_data_s* data, void* ptr[])
+{
+	data->ops->zhardthresh(data->size, (*(unsigned int*)data->data_ptr), ptr[0], ptr[1]);
+}
+
+/**
+ * Hard thresholding (with strides)
+ *
+ * y = HT(x, k), selects k largest elements of x
+ * computes y = x * (abs(x) > t(k)), 
+ * k = threshold index of sorted x, t(k)= value of sorted x at k
+ *
+ * @param D number of dimensions
+ * @param dims dimensions of input/output
+ * @param k threshold (sorted) index
+ * @param flags jointly thresholded dimensions
+ * @param tmp_norm temporary array for joint operation
+ * @param ostrs destination strides
+ * @param optr destination -- thresholded values
+ * @param istrs source strides
+ * @param iptr source -- values to be thresholded
+ */
+void md_zhardthresh2(unsigned int D, const long dims[D], unsigned int k, unsigned int flags, const long ostrs[D], complex float* optr, const long istrs[D], const complex float* iptr)
+{
+	if (0 == flags) {
+
+		optimized_twoop_oi(D, dims, ostrs, optr, istrs, iptr, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh, &k);
+		return;
+	}
+
+	long norm_dims[D];
+	md_select_dims(D, ~flags, norm_dims, dims);
+
+	complex float* tmp_norm = md_alloc_sameplace(D, norm_dims, CFL_SIZE, iptr);
+
+	md_zhardthresh_joint2(D, dims, k, flags, tmp_norm, ostrs, optr, istrs,iptr);
+
+	md_free(tmp_norm);
+}
+
+/**
+ * Hard thresholding (without strides)
+ *
+ * y = HT(x, k), select k largest elements.
+ *
+ * @param D number of dimensions
+ * @param dims dimensions of input/output
+ * @param k threshold parameter
+ * @param flags jointly thresholded dimensions
+ * @param optr destination -- thresholded values
+ * @param iptr source -- values to be thresholded
+ */
+void md_zhardthresh(unsigned int D, const long dims[D], unsigned int k, unsigned int flags, complex float* optr, const complex float* iptr)
+{
+	long strs[D];
+	md_calc_strides(D, strs, dims, CFL_SIZE);
+
+	md_zhardthresh2(D, dims, k, flags, strs, optr, strs, iptr);
+
+}
 
 /**
  * Elementwise minimum of input and scalar (with strides)
