@@ -1,11 +1,13 @@
 /* Copyright 2013-2015 The Regents of the University of California.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2017. Martin Uecker.
+ * Copyright 2017. Intel Corporation.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2012-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2013      Frank Ong <frankong@berkeley.edu>
+ * 2017      Michael J. Anderson <michael.j.anderson@intel.com>
  *
  * Generic operations on multi-dimensional arrays. Most functions
  * come in two flavours:
@@ -85,27 +87,58 @@ void md_parallel_nary(unsigned int C, unsigned int D, const long dim[D], unsigne
 		return;
 	}
 
-	int b = ffsl(flags & -flags) - 1;
-	assert(MD_IS_SET(flags, b));
-
-	flags = MD_CLEAR(flags, b);
-
 	long dimc[D];
-	md_select_dims(D, ~MD_BIT(b), dimc, dim);
+	md_select_dims(D, ~flags, dimc, dim);
+
+	// Collect all parallel dimensions
+	int nparallel = 0;
+	int parallel_b[D];
+
+	long parallel_dim[D];
+	long total_iterations = 1L;
+
+	while (0 != flags) {
+
+		int b = ffsl(flags & -flags) - 1;
+
+		assert(MD_IS_SET(flags, b));
+
+		flags = MD_CLEAR(flags, b);
+
+		debug_printf(DP_DEBUG4, "Parallelize: %d\n", dim[b]);
+
+		parallel_b[nparallel] = b;
+		parallel_dim[nparallel] = dim[b];
+
+		total_iterations *= parallel_dim[nparallel];
+		nparallel++;
+	}
 
-	debug_printf(DP_DEBUG4, "Parallelize: %d\n", dim[b]);
 
-	// FIXME: this probably doesn't nest
-	// (maybe collect all parallelizable dims into one giant loop?)
 	#pragma omp parallel for
-	for (long i = 0; i < dim[b]; i++) {
+	for (long i = 0; i < total_iterations; i++) {
+
+		// Recover place in parallel iteration space
+		long iter_i[D];
+		long ii = i;
+
+		for (int p = nparallel - 1; p >= 0; p--) {
+
+			iter_i[p] = ii % parallel_dim[p];
+			ii /= parallel_dim[p];
+		}
 
 		void* moving_ptr[C];
 
-		for (unsigned int j = 0; j < C; j++)
-			moving_ptr[j] = ptr[j] + i * str[j][b];
+		for (unsigned int j = 0; j < C; j++) {
 
-		md_parallel_nary(C, D, dimc, flags, str, moving_ptr, data, fun);
+			moving_ptr[j] = ptr[j];
+
+			for(int p = 0; p < nparallel; p++)
+				moving_ptr[j] += iter_i[p] * str[j][parallel_b[p]];
+		}
+
+		md_nary(C, D, dimc, str, moving_ptr, data, fun);
 	}
 }
 
@@ -356,6 +389,22 @@ bool md_check_dimensions(unsigned int N, const long dims[N], unsigned int flags)
 
 
 
+/*
+ * compute non-trivial (> 1) dims
+ */
+unsigned long md_nontriv_dims(unsigned int D, const long dims[D])
+{
+	unsigned long flags = 0;
+
+	for (unsigned int i = 0; i < D; i++)
+		if (dims[i] > 1)
+			flags = MD_SET(flags, i);
+
+	return flags;
+}
+
+
+
 /**
  * Set all dimensions to one
  *
@@ -446,6 +495,25 @@ void md_min_dims(unsigned int D, unsigned long flags, long odims[D], const long
 }
 
 
+/**
+ * Set the output's flagged dimensions to the maximum of the two input dimensions
+ *
+ * odims = [ MAX(idims1[0],idims2[0] ... MAX(idims1[D-1],idims2[D-1]) ]
+ *
+ * @param D number of dimensions
+ * @param flags bitmask specifying which dimensions to maximize
+ * @param odims output dimensions
+ * @param idims1 input 1 dimensions
+ * @param idims2 input 2 dimensions
+ */
+void md_max_dims(unsigned int D, unsigned long flags, long odims[D], const long idims1[D], const long idims2[D])
+{
+	for (unsigned int i = 0; i < D; i++)
+		if (MD_IS_SET(flags, i))
+			odims[i] = MAX(idims1[i], idims2[i]);
+}
+
+
 
 struct data_s {
 
@@ -483,7 +551,16 @@ void md_clear2(unsigned int D, const long dim[D], const long str[D], void* ptr,
 #else
 	struct data_s data = { size };
 #endif
-	optimized_nop(1, MD_BIT(0), D, dim, nstr, (void*[1]){ ptr }, (size_t[1]){ size }, nary_clear, &data);
+	unsigned long flags = 0;
+
+	for (unsigned int i = 0; i < D; i++)
+		if (0 == str[i])
+			flags |= MD_BIT(i);
+
+	long dim2[D];
+	md_select_dims(D, ~flags, dim2, dim);
+
+	optimized_nop(1, MD_BIT(0), D, dim2, nstr, (void*[1]){ ptr }, (size_t[1]){ size }, nary_clear, &data);
 }
 
 
@@ -1488,6 +1565,8 @@ void md_circ_shift2(unsigned int D, const long dimensions[D], const long center[
 	dim1[i] = shift;
 	dim2[i] = dimensions[i] - shift;
 
+	assert((dim1[i] >= 0) && (dim2[i] >= 0));
+
 	pos[i] = 0;
 
 	//printf("%d: %ld %ld %d\n", i, dim1[i], dim2[i], sizeof(dimensions));
