@@ -1,10 +1,10 @@
 /* Copyright 2015. The Regents of the University of California.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2017. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2013-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2013-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014 Jonathan Tamir <jtamir@eecs.berkeley.edu>
  * 2014 Frank Ong <frankong@berkeley.edu>
  *
@@ -240,7 +240,9 @@ struct operator_p_s {
 
 const struct operator_p_s* operator_p_ref(const struct operator_p_s* x)
 {
-	operator_ref(&x->op);
+	if (NULL != x)
+		operator_ref(&x->op);
+
 	return x;
 }
 
@@ -271,7 +273,8 @@ const struct iovec_s* operator_p_codomain(const struct operator_p_s* op)
 
 void operator_p_free(const struct operator_p_s* x)
 {
-	operator_free(&x->op);
+	if (NULL != x)
+		operator_free(&x->op);
 }
 
 
@@ -284,7 +287,7 @@ struct op_p_data_s {
 	operator_del_t del;
 };
 
-DEF_TYPEID(op_p_data_s);
+static DEF_TYPEID(op_p_data_s);
 
 static void op_p_apply(const operator_data_t* _data, unsigned int N, void* args[N])
 {
@@ -364,6 +367,14 @@ const struct operator_p_s* operator_p_create(unsigned int ON, const long out_dim
 }
 
 
+
+const struct operator_s* operator_p_upcast(const struct operator_p_s* op)
+{
+	return &op->op;
+}
+
+
+
 struct identity_s {
 
 	INTERFACE(operator_data_t);
@@ -372,7 +383,7 @@ struct identity_s {
 	const struct iovec_s* codomain;
 };
 
-DEF_TYPEID(identity_s);
+static DEF_TYPEID(identity_s);
 
 static void identity_apply(const operator_data_t* _data, unsigned int N, void* args[N])
 {
@@ -385,6 +396,7 @@ static void identity_apply(const operator_data_t* _data, unsigned int N, void* a
 static void identity_free(const operator_data_t* _data)
 {
         const struct identity_s* d = CAST_DOWN(identity_s, _data);
+        iovec_free(d->domain);
         iovec_free(d->codomain);
 	free((void*)d);
 }
@@ -423,7 +435,7 @@ struct operator_chain_s {
 	const struct operator_s* b;
 };
 
-DEF_TYPEID(operator_chain_s);
+static DEF_TYPEID(operator_chain_s);
 
 
 static void chain_apply(const operator_data_t* _data, unsigned int N, void* args[N])
@@ -469,6 +481,10 @@ const struct operator_s* operator_chain(const struct operator_s* a, const struct
 	
 	// check compatibility
 
+	debug_printf(DP_DEBUG4, "operator chain:\n");
+	debug_print_dims(DP_DEBUG4, a->domain[0]->N, a->domain[0]->dims);
+	debug_print_dims(DP_DEBUG4, b->domain[1]->N, b->domain[1]->dims);
+
 	assert((2 == a->N) && (2 == b->N));
 	assert((MD_BIT(0) == a->io_flags) && (MD_BIT(0) == b->io_flags));
 	assert(a->domain[0]->N == b->domain[1]->N);
@@ -477,7 +493,7 @@ const struct operator_s* operator_chain(const struct operator_s* a, const struct
 	// check whether intermediate storage can be simple
 
 	assert(a->domain[0]->N == md_calc_blockdim(a->domain[0]->N, a->domain[0]->dims, a->domain[0]->strs, a->domain[0]->size));
-	assert(b->domain[1]->N == md_calc_blockdim(b->domain[1]->N, b->domain[1]->dims, b->domain[1]->strs, a->domain[1]->size));
+	assert(b->domain[1]->N == md_calc_blockdim(b->domain[1]->N, b->domain[1]->dims, b->domain[1]->strs, b->domain[1]->size));
 
 	c->a = operator_ref(a);
 	c->b = operator_ref(b);
@@ -514,7 +530,7 @@ struct operator_stack_s {
 	long src_offset;
 };
 
-DEF_TYPEID(operator_stack_s);
+static DEF_TYPEID(operator_stack_s);
 
 
 static void stack_apply(const operator_data_t* _data, unsigned int N, void* args[N])
@@ -651,23 +667,94 @@ void operator_p_apply_unchecked(const struct operator_p_s* op, float mu, complex
 }
 
 
-void operator_iter(void* o, float* _dst, const float* _src)
-{
-	complex float* dst = (complex float*)_dst;
-	const complex float* src = (complex float*)_src;
+struct op_bind_s {
 
-	operator_apply_unchecked(o, dst, src);
+	INTERFACE(operator_data_t);
+
+	unsigned int D;
+	unsigned int arg;
+
+	const struct operator_s* op;
+	void* ptr;
+};
+
+static DEF_TYPEID(op_bind_s);
+
+static void op_bind_apply(const operator_data_t* _data, unsigned int N, void* args[N])
+{
+	const struct op_bind_s* data = CAST_DOWN(op_bind_s, _data);
+	assert(data->D == N + 1);
+
+	void* n_args[N + 1];
+
+	for (unsigned int i = 0, j = 0; i < N; i++, j++) {
+
+		// insert bound argument
+
+		if (data->arg == i)
+			n_args[j++] = data->ptr;
+
+		n_args[j] = args[i];
+	}
+
+	operator_generic_apply_unchecked(data->op, N + 1, n_args);
 }
 
-void operator_p_iter(void* o, float lambda, float* _dst, const float* _src)
+static void op_bind_del(const operator_data_t* _data)
 {
-	complex float* dst = (complex float*)_dst;
-	const complex float* src = (complex float*)_src;
-
-	operator_p_apply_unchecked(o, lambda, dst, src);
+	const struct op_bind_s* data = CAST_DOWN(op_bind_s, _data);
+	operator_free(data->op);
 }
 
 
+/**
+ * Create a new operator that binds argument 'arg'.
+ */
+const struct operator_s* operator_bind2(const struct operator_s* op, unsigned int arg,
+			unsigned int N, const long dims[N], const long strs[N], void* ptr)
+{
+	unsigned int D = operator_nr_args(op);
+	assert(arg < D);
+	assert(!MD_IS_SET(op->io_flags, arg));
+	assert(iovec_check(operator_arg_domain(op, arg), N, dims, strs));
+
+	unsigned int nn[D - 1];
+	const long* ndims[D - 1];
+	const long* nstrs[D - 1];
+
+	unsigned int n_flags = 0u;
+
+	for (unsigned int i = 0, j = 0; i < D; i++) {
+
+		if (arg == i)
+			continue;
+
+		nn[j] = operator_arg_domain(op, i)->N;
+		ndims[j] = operator_arg_domain(op, i)->dims;
+		nstrs[j] = operator_arg_domain(op, i)->strs;
+
+		if (MD_IS_SET(op->io_flags, i))
+			n_flags |= MD_BIT(j);
+
+		j++;
+	}
+
+	PTR_ALLOC(struct op_bind_s, data);
+	SET_TYPEID(op_bind_s, data);
+
+	data->D = D;
+	data->arg = arg;
+	data->ptr = ptr;
+	data->op = op;
+
+	return operator_generic_create2(D - 1, n_flags,
+		nn, ndims, nstrs,
+		CAST_UP(PTR_PASS(data)), op_bind_apply, op_bind_del);
+}
+
+
+
+
 
 struct op_loop_s {
 
@@ -681,7 +768,7 @@ struct op_loop_s {
 	const struct operator_s* op;
 };
 
-DEF_TYPEID(op_loop_s);
+static DEF_TYPEID(op_loop_s);
 
 static void op_loop_del(const operator_data_t* _data)
 {
@@ -809,7 +896,7 @@ struct copy_data_s {
 	const long** strs;
 };
 
-DEF_TYPEID(copy_data_s);
+static DEF_TYPEID(copy_data_s);
 
 static void copy_fun(const operator_data_t* _data, unsigned int N, void* args[N])
 {
@@ -902,7 +989,7 @@ struct gpu_data_s {
 	const struct operator_s* op;
 };
 
-DEF_TYPEID(gpu_data_s);
+static DEF_TYPEID(gpu_data_s);
 
 static void gpuwrp_fun(const operator_data_t* _data, unsigned int N, void* args[N])
 {
