@@ -1,5 +1,5 @@
 /* Copyright 2014-2015. The Regents of the University of California.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2017. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  */
@@ -63,6 +63,9 @@ extern const struct operator_s* operator_chainN(unsigned int N, const struct ope
 //extern const struct operator_s* operator_sum(const struct operator_s* a, const struct operator_s* b);
 extern const struct operator_s* operator_stack(unsigned int D, unsigned int E, const struct operator_s* a, const struct operator_s* b);
 
+extern const struct operator_s* operator_bind2(const struct operator_s* op, unsigned int arg,
+			unsigned int N, const long dims[__VLA(N)], const long strs[__VLA(N)], void* ptr);
+
 // del functions
 extern void operator_free(const struct operator_s* x);
 extern void operator_p_free(const struct operator_p_s* x);
@@ -97,6 +100,9 @@ extern operator_data_t* operator_get_data(const struct operator_s* op);
 extern operator_data_t* operator_p_get_data(const struct operator_p_s* x);
 
 
+extern const struct operator_s* operator_p_upcast(const struct operator_p_s* op);
+
+
 extern const struct operator_s* operator_copy_wrapper(unsigned int N, const long* strs[N], const struct operator_s* op);
 
 extern const struct operator_s* operator_gpu_wrapper(const struct operator_s* op);
@@ -116,9 +122,6 @@ extern const struct operator_s* operator_loop2(unsigned int N, const unsigned in
 extern const struct operator_s* operator_loop(unsigned int D, const long dims[D], const struct operator_s* op);
 
 
-// iter helper functions
-extern void operator_iter(void* _o, float* _dst, const float* _src);
-extern void operator_p_iter(void* _o, float lambda, float* _dst, const float* _src);
 
 #include "misc/cppwrap.h"
 
