@@ -1,10 +1,10 @@
 /* Copyright 2014. The Regents of the University of California.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2017. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2013-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2013-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  *
  * 
  * Optimization framework for operations on multi-dimensional arrays.
@@ -265,7 +265,8 @@ static bool split_dims(unsigned int D, unsigned int N, long dims[N + 1], long (*
 }
 
 
-unsigned int optimize_dims(unsigned int D, unsigned int N, long dims[N], long (*strs[D])[N])
+
+unsigned int simplify_dims(unsigned int D, unsigned int N, long dims[N], long (*strs[D])[N])
 {
 	merge_dims(D, N, dims, strs);
 
@@ -281,6 +282,14 @@ unsigned int optimize_dims(unsigned int D, unsigned int N, long dims[N], long (*
 		ND = 1;
 	}
 
+	return ND;
+}
+
+
+unsigned int optimize_dims(unsigned int D, unsigned int N, long dims[N], long (*strs[D])[N])
+{
+	unsigned int ND = simplify_dims(D, N, dims, strs);
+
 	debug_print_dims(DP_DEBUG4, ND, dims);
 
 	float blocking[N];
@@ -442,7 +451,8 @@ static unsigned long parallelizable(unsigned int D, unsigned int io, unsigned in
 }
 
 
-#define CHUNK (32 * 1024)
+extern long num_chunk_size;
+long num_chunk_size = 32 * 1024;
 
 
 /**
@@ -465,7 +475,7 @@ unsigned long dims_parallel(unsigned int D, unsigned int io, unsigned int N, con
 
 			reps /= dims[i];
 
-			if (reps < CHUNK)
+			if (reps < num_chunk_size)
 				break;
 
 			oflags = MD_SET(oflags, i);
@@ -519,6 +529,8 @@ static void nary_opt(void* _data, void* ptr[])
 }
 
 
+
+
 /**
  * Optimized n-op.
  *
@@ -534,6 +546,25 @@ static void nary_opt(void* _data, void* ptr[])
  */
 void optimized_nop(unsigned int N, unsigned int io, unsigned int D, const long dim[D], const long (*nstr[N])[D], void* const nptr[N], size_t sizes[N], md_nary_opt_fun_t too, void* data_ptr)
 {
+	assert(N > 0);
+
+	if (0 == D) {
+
+		long dim1[1] = { 1 };
+		long tstrs[N][1];
+		long (*nstr1[N])[1];
+
+		for (unsigned int i = 0; i < N; i++) {
+
+			tstrs[i][0] = 0;
+			nstr1[i] = &tstrs[i];
+		}
+
+		optimized_nop(N, io, 1, dim1, (void*)nstr1, nptr, sizes, too, data_ptr);
+
+		return;
+	}
+
 	long tdims[D];
 	md_copy_dims(D, tdims, dim);
 
@@ -544,6 +575,7 @@ void optimized_nop(unsigned int N, unsigned int io, unsigned int D, const long d
 	for (unsigned int i = 0; i < N; i++) {
 
 		md_copy_strides(D, tstrs[i], *nstr[i]);
+
 		nstr1[i] = &tstrs[i];
 		nptr1[i] = nptr[i];
 	}
@@ -575,7 +607,7 @@ void optimized_nop(unsigned int N, unsigned int io, unsigned int D, const long d
 		nstr2[i] = *nstr1[i] + skip;
 
 #ifdef USE_CUDA
-	debug_printf(DP_DEBUG3, "This is a %s call\n.", use_gpu(N, nptr1) ? "gpu" : "cpu");
+	debug_printf(DP_DEBUG4, "This is a %s call\n.", use_gpu(N, nptr1) ? "gpu" : "cpu");
 
 	struct nary_opt_data_s data = { md_calc_size(skip, tdims), use_gpu(N, nptr1) ? &gpu_ops : &cpu_ops, data_ptr };
 #else
