@@ -28,7 +28,7 @@ static const char help_str[] = "Create an array filled with ones with {dims} dim
 
 int main_ones(int argc, char* argv[])
 {
-	mini_cmdline(argc, argv, -3, usage_str, help_str);
+	mini_cmdline(&argc, argv, -3, usage_str, help_str);
 
 	num_init();
 
