@@ -32,7 +32,7 @@ static const char help_str[] = "Calculates root of sum of squares along selected
 
 int main_rss(int argc, char* argv[argc])
 {
-	mini_cmdline(argc, argv, 3, usage_str, help_str);
+	mini_cmdline(&argc, argv, 3, usage_str, help_str);
 
 	num_init();
 
