@@ -33,6 +33,7 @@
 
 #include "num/multind.h"
 #include "num/flpmath.h"
+#include "num/iovec.h"
 
 #include "linops/linop.h"
 #include "linops/sampling.h"
@@ -58,6 +59,7 @@ const struct sense_conf sense_defaults = {
 	.rwiter = 1,
 	.gamma = -1.,
 	.cclambda = 0.,
+	.bpsense = false,
 };
 
 
@@ -120,20 +122,24 @@ const struct operator_s* sense_recon_create(const struct sense_conf* conf, const
 		  unsigned int num_funs,
 		  const struct operator_p_s* thresh_op[num_funs],
 		  const struct linop_s* thresh_funs[num_funs],
-		  const long ksp_dims[DIMS],
 		  const struct operator_s* precond_op)
 {
 	struct lsqr_conf lsqr_conf = { conf->cclambda, conf->gpu };
 
 	const struct operator_s* op = NULL;
 
+	assert(DIMS == linop_domain(sense_op)->N);
 
 	long img_dims[DIMS];
-	md_select_dims(DIMS, ~COIL_FLAG, img_dims, dims);
+	md_copy_dims(DIMS, img_dims, linop_domain(sense_op)->dims);
 
+	long ksp_dims[DIMS];
+	md_copy_dims(DIMS, ksp_dims, linop_codomain(sense_op)->dims);
 
 	if (conf->rvc) {
 
+		assert(!conf->bpsense); // FIXME: add rvc as separate constraint or build into forward model earlier
+
 		struct linop_s* rvc = linop_realval_create(DIMS, img_dims);
 		struct linop_s* tmp_op = linop_chain(rvc, sense_op);
 
@@ -144,6 +150,8 @@ const struct operator_s* sense_recon_create(const struct sense_conf* conf, const
 
 	if (1 < conf->rwiter) {
 
+		assert(!conf->bpsense); // not compatible
+
 		struct linop_s* sampling = linop_sampling_create(dims, pat_dims, pattern);
 		struct linop_s* tmp_op = linop_chain(sense_op, sampling);
 
@@ -163,11 +171,17 @@ const struct operator_s* sense_recon_create(const struct sense_conf* conf, const
 	} else
 	if (NULL == pattern) {
 
-		op = lsqr2_create(&lsqr_conf, italgo, iconf, (const float*)init, sense_op, precond_op,
-					num_funs, thresh_op, thresh_funs);
+		if (conf->bpsense)
+			op = lsqr2_create(&lsqr_conf, italgo, iconf, (const float*)init, NULL, NULL,
+					num_funs, thresh_op, thresh_funs, NULL);
+		else
+			op = lsqr2_create(&lsqr_conf, italgo, iconf, (const float*)init, sense_op, precond_op,
+					num_funs, thresh_op, thresh_funs, NULL);
 
 	} else {
 
+		assert(!conf->bpsense); // pattern should be built into forward model
+
 		complex float* weights = md_alloc(DIMS, pat_dims, CFL_SIZE);
 #if 0
 		// buggy
@@ -179,11 +193,12 @@ const struct operator_s* sense_recon_create(const struct sense_conf* conf, const
 #endif
 		// FIXME: weights is never freed
 
-		struct linop_s* weights_op = linop_cdiag_create(DIMS, ksp_dims, FFT_FLAGS, weights);	// FIXME: check pat_dims
+		struct linop_s* weights_op = linop_cdiag_create(DIMS, ksp_dims, ~COIL_FLAG, weights);
 
 		op = wlsqr2_create(&lsqr_conf, italgo, iconf, (const float*)init,
 						sense_op, weights_op, precond_op,
-						num_funs, thresh_op, thresh_funs);
+						num_funs, thresh_op, thresh_funs,
+						NULL);
 	}
 
 	return op;
