@@ -27,6 +27,7 @@ struct sense_conf {
 	int rwiter;	// should be moved into a recon_lad
 	float gamma;	// ..
 	float cclambda;
+	bool bpsense;
 };
 
 
@@ -43,7 +44,6 @@ extern const struct operator_s* sense_recon_create(const struct sense_conf* conf
 		  unsigned int num_funs,
 		  const struct operator_p_s* thresh_op[num_funs],
 		  const struct linop_s* thresh_funs[num_funs],
-		  const long ksp_dims[DIMS],
 		  const struct operator_s* precond_op);
 
 extern void debug_print_sense_conf(int debug_level, const struct sense_conf* conf);
