@@ -29,7 +29,7 @@ static const char help_str[] = "Raise array to the power of {exponent}. The expo
 
 int main_spow(int argc, char* argv[argc])
 {
-	mini_cmdline(argc, argv, 3, usage_str, help_str);
+	mini_cmdline(&argc, argv, 3, usage_str, help_str);
 
 	num_init();
 
