@@ -29,7 +29,7 @@ static const char help_str[] = "Remove singleton dimensions of array.\n";
 
 int main_squeeze(int argc, char* argv[])
 {
-	mini_cmdline(argc, argv, 2, usage_str, help_str);
+	mini_cmdline(&argc, argv, 2, usage_str, help_str);
 
 	num_init();
 
