@@ -1,10 +1,10 @@
 /* Copyright 2014. The Regents of the University of California.
- * Copyright 2015-2016. Martin Uecker.
+ * Copyright 2015-2017. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  * 
  * Authors:
- * 2013-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2013-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2015-2016 Jon Tamir <jtamir@eecs.berkeley.edu>
  * 2015 Frank Ong <frankong@berkeley.edu>
  */
@@ -21,6 +21,7 @@
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
+#include "misc/debug.h"
 #include "misc/opts.h"
 
 #include "lowrank/lrthresh.h"
@@ -43,12 +44,18 @@ static void wthresh(unsigned int D, const long dims[D], float lambda, unsigned i
 	md_singleton_dims(D, minsize);
 
 	long course_scale[3] = MD_INIT_ARRAY(3, 16);
-	md_min_dims(3, ~0u, minsize, dims, course_scale);
+	md_copy_dims(3, minsize, course_scale);
+
+	unsigned int wflags = 7; // FIXME
+
+	for (unsigned int i = 0; i < 3; i++)
+		if (dims[i] < minsize[i])
+			wflags = MD_CLEAR(wflags, i);
 
 	long strs[D];
 	md_calc_strides(D, strs, dims, CFL_SIZE);
 
-	const struct linop_s* w = linop_wavelet3_create(D, 7, dims, strs, minsize);
+	const struct linop_s* w = linop_wavelet_create(D, wflags, dims, strs, minsize, false);
 	const struct operator_p_s* p = prox_unithresh_create(D, w, lambda, flags, false);
 
 	operator_p_apply(p, 1., D, dims, out, D, dims, in);
@@ -158,6 +165,7 @@ int main_threshold(int argc, char* argv[])
 
 		case HARD:
 			hard_thresh(N, dims, lambda, odata, idata);
+			break;
 
 		default:
 			md_zsoftthresh(N, dims, lambda, flags, odata, idata);
