@@ -39,7 +39,7 @@
 #include "sense/model.h"
 #include "sense/optcom.h"
 
-#include "wavelet3/wavthresh.h"
+#include "wavelet/wavthresh.h"
 
 
 // create wavecaipi operator
@@ -159,7 +159,7 @@ int main_wave(int argc, char* argv[])
 		minsize[1] = MIN(img_dims[1], 16);
 		minsize[2] = MIN(img_dims[2], 16);
 
-		thresh_op = prox_wavelet3_thresh_create(DIMS, img_dims, FFT_FLAGS, minsize, lambda, randshift);
+		thresh_op = prox_wavelet_thresh_create(DIMS, img_dims, FFT_FLAGS, 0u, minsize, lambda, randshift);
 	}
     
 
