@@ -27,7 +27,7 @@ static const char help_str[] = "Create a zero-filled array with {dims} dimension
 
 int main_zeros(int argc, char* argv[])
 {
-	mini_cmdline(argc, argv, -3, usage_str, help_str);
+	mini_cmdline(&argc, argv, -3, usage_str, help_str);
 
 	num_init();
 
